#ifndef EFL_GENERATED_ECORE_AUDIO_IN_HH
#define EFL_GENERATED_ECORE_AUDIO_IN_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "ecore_audio_in.eo.h"
}

#include "ecore_audio.eo.hh"

struct ecore_audio_in;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct ecore_audio_in
{
   /// @brief Read from the input
   ///
   /// @since 1.8
   ///
   /// @param buf 
   /// @param len 
   ///
   ssize_t read(void * buf_, size_t len_) const;

   /// @brief Internal read function
   ///
   /// @since 1.8
   ///
   /// @param buf 
   /// @param len 
   ///
   ssize_t read_internal(void * buf_, size_t len_) const;

   /// @brief Seek within the input
   ///
   /// @since 1.8
   ///
   /// @param offs 
   /// @param mode 
   ///
   double seek(double offs_, int mode_) const;

   /// @brief Get the playback speed of the input.
   ///
   /// @since 1.8
   ///
   /// @param speed 
   ///
   double speed_get() const;

   /// @brief Set the playback speed of the input.
   ///
   /// @since 1.8
   ///
   /// @param speed 
   ///
   void speed_set(double speed_) const;

   /// @brief Get the he sample-rate of the input
   ///
   /// @since 1.8
   ///
   /// @param samplerate 
   ///
   int samplerate_get() const;

   /// @brief Set the sample-rate of the input
   ///
   /// @since 1.8
   ///
   /// @param samplerate 
   ///
   void samplerate_set(int samplerate_) const;

   /// @brief Get the amount of channels the input has
   ///
   /// @since 1.8
   ///
   /// @param channels 
   ///
   int channels_get() const;

   /// @brief Set the amount of channels the input has
   ///
   /// @since 1.8
   ///
   /// @param channels 
   ///
   void channels_set(int channels_) const;

   /// @brief Get the the preloaded state of the input
   ///
   /// @since 1.8
   ///
   /// @param preloaded 
   ///
   bool preloaded_get() const;

   /// @brief Set the preloaded state of the input
   ///
   /// @since 1.8
   ///
   /// @param preloaded 
   ///
   void preloaded_set(bool preloaded_) const;

   /// @brief Get the looped state of the input
   ///
   /// @since 1.8
   ///
   /// @param looped 
   ///
   bool looped_get() const;

   /// @brief Set the looped state of the input
   ///
   /// If the input is looped and reaches the end it will start from the
   /// beginning again. At the same time the event @ref ECORE_AUDIO_EV_IN_LOOPED
   /// will be emitted
   ///
   /// @since 1.8
   ///
   /// @param looped 
   ///
   void looped_set(bool looped_) const;

   /// @brief Get the length of the input
   ///
   /// @since 1.8
   ///
   /// @param length 
   ///
   double length_get() const;

   /// @brief Set the length of the input
   ///
   /// This function is only implemented by some classes
   /// (i.e. ECORE_AUDIO_OBJ_IN_TONE_CLASS)
   ///
   /// @since 1.8
   ///
   /// @param length 
   ///
   void length_set(double length_) const;

   /// @brief Get the output that this input is attached to
   ///
   /// @since 1.8
   ///
   /// @param output 
   ///
   ::efl::eo::concrete output_get() const;

   /// @brief Get the remaining time of the input
   ///
   /// @since 1.8
   ///
   /// @param remaining 
   ///
   double remaining_get() const;

   /// Called when an input has looped.
   template <typename F>
   ::efl::eo::signal_connection
   callback_in_looped_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (ECORE_AUDIO_IN_EVENT_IN_LOOPED, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         ECORE_AUDIO_IN_EVENT_IN_LOOPED );
   }

   /// Called when an input has looped.
   template <typename T>
   void
   callback_in_looped_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (ECORE_AUDIO_IN_EVENT_IN_LOOPED, info));
   }

   /// Called when an input has stopped playing.
   template <typename F>
   ::efl::eo::signal_connection
   callback_in_stopped_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (ECORE_AUDIO_IN_EVENT_IN_STOPPED, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         ECORE_AUDIO_IN_EVENT_IN_STOPPED );
   }

   /// Called when an input has stopped playing.
   template <typename T>
   void
   callback_in_stopped_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (ECORE_AUDIO_IN_EVENT_IN_STOPPED, info));
   }

   /// Called when the input samplerate has changed.
   template <typename F>
   ::efl::eo::signal_connection
   callback_in_samplerate_changed_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (ECORE_AUDIO_IN_EVENT_IN_SAMPLERATE_CHANGED, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         ECORE_AUDIO_IN_EVENT_IN_SAMPLERATE_CHANGED );
   }

   /// Called when the input samplerate has changed.
   template <typename T>
   void
   callback_in_samplerate_changed_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (ECORE_AUDIO_IN_EVENT_IN_SAMPLERATE_CHANGED, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(ECORE_AUDIO_IN_CLASS);
   }

   operator ::ecore_audio_in() const;
   operator ::ecore_audio_in&();
   operator ::ecore_audio_in const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::ecore_audio_in*() const { return static_cast<::ecore_audio_in*>(static_cast<D const*>(this)->p); }
      operator ::ecore_audio_in const*() const { return static_cast<::ecore_audio_in const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::ecore_audio_in const*() const { return static_cast<::ecore_audio_in const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class ecore_audio_in
struct ecore_audio_in
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(ecore_audio)
      , EO_CXX_INHERIT(eo::base)
{
   //@{
   /**
      @brief Constructs a new ecore_audio_in object.

      Constructs a new ecore_audio_in object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      ecore_audio_in my_ecore_audio_in(efl::eo::parent = parent_object);
      @endcode

      @see ecore_audio_in(Eo* eo)
   */
   explicit ecore_audio_in(::efl::eo::parent_type _p)
      : ecore_audio_in(_ctors_call(_p))
   {}

   explicit ecore_audio_in()
      : ecore_audio_in(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit ecore_audio_in(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit ecore_audio_in(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   ecore_audio_in(ecore_audio_in const& other)
      : ecore_audio_in(eo_ref(other._eo_ptr()))
   {}

   ~ecore_audio_in() {}

   /// @brief Read from the input
   ///
   /// @since 1.8
   ///
   /// @param buf 
   /// @param len 
   ///
   ssize_t read(void * buf_, size_t len_) const;

   /// @brief Internal read function
   ///
   /// @since 1.8
   ///
   /// @param buf 
   /// @param len 
   ///
   ssize_t read_internal(void * buf_, size_t len_) const;

   /// @brief Seek within the input
   ///
   /// @since 1.8
   ///
   /// @param offs 
   /// @param mode 
   ///
   double seek(double offs_, int mode_) const;

   /// @brief Get the playback speed of the input.
   ///
   /// @since 1.8
   ///
   /// @param speed 
   ///
   double speed_get() const;

   /// @brief Set the playback speed of the input.
   ///
   /// @since 1.8
   ///
   /// @param speed 
   ///
   void speed_set(double speed_) const;

   /// @brief Get the he sample-rate of the input
   ///
   /// @since 1.8
   ///
   /// @param samplerate 
   ///
   int samplerate_get() const;

   /// @brief Set the sample-rate of the input
   ///
   /// @since 1.8
   ///
   /// @param samplerate 
   ///
   void samplerate_set(int samplerate_) const;

   /// @brief Get the amount of channels the input has
   ///
   /// @since 1.8
   ///
   /// @param channels 
   ///
   int channels_get() const;

   /// @brief Set the amount of channels the input has
   ///
   /// @since 1.8
   ///
   /// @param channels 
   ///
   void channels_set(int channels_) const;

   /// @brief Get the the preloaded state of the input
   ///
   /// @since 1.8
   ///
   /// @param preloaded 
   ///
   bool preloaded_get() const;

   /// @brief Set the preloaded state of the input
   ///
   /// @since 1.8
   ///
   /// @param preloaded 
   ///
   void preloaded_set(bool preloaded_) const;

   /// @brief Get the looped state of the input
   ///
   /// @since 1.8
   ///
   /// @param looped 
   ///
   bool looped_get() const;

   /// @brief Set the looped state of the input
   ///
   /// If the input is looped and reaches the end it will start from the
   /// beginning again. At the same time the event @ref ECORE_AUDIO_EV_IN_LOOPED
   /// will be emitted
   ///
   /// @since 1.8
   ///
   /// @param looped 
   ///
   void looped_set(bool looped_) const;

   /// @brief Get the length of the input
   ///
   /// @since 1.8
   ///
   /// @param length 
   ///
   double length_get() const;

   /// @brief Set the length of the input
   ///
   /// This function is only implemented by some classes
   /// (i.e. ECORE_AUDIO_OBJ_IN_TONE_CLASS)
   ///
   /// @since 1.8
   ///
   /// @param length 
   ///
   void length_set(double length_) const;

   /// @brief Get the output that this input is attached to
   ///
   /// @since 1.8
   ///
   /// @param output 
   ///
   ::efl::eo::concrete output_get() const;

   /// @brief Get the remaining time of the input
   ///
   /// @since 1.8
   ///
   /// @param remaining 
   ///
   double remaining_get() const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }

   /// Called when an input has looped.
   template <typename F>
   ::efl::eo::signal_connection
   callback_in_looped_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (ECORE_AUDIO_IN_EVENT_IN_LOOPED, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         ECORE_AUDIO_IN_EVENT_IN_LOOPED );
   }

   /// Called when an input has looped.
   template <typename T>
   void
   callback_in_looped_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (ECORE_AUDIO_IN_EVENT_IN_LOOPED, info));
   }

   /// Called when the input samplerate has changed.
   template <typename F>
   ::efl::eo::signal_connection
   callback_in_samplerate_changed_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (ECORE_AUDIO_IN_EVENT_IN_SAMPLERATE_CHANGED, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         ECORE_AUDIO_IN_EVENT_IN_SAMPLERATE_CHANGED );
   }

   /// Called when the input samplerate has changed.
   template <typename T>
   void
   callback_in_samplerate_changed_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (ECORE_AUDIO_IN_EVENT_IN_SAMPLERATE_CHANGED, info));
   }

   /// Called when an input has stopped playing.
   template <typename F>
   ::efl::eo::signal_connection
   callback_in_stopped_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (ECORE_AUDIO_IN_EVENT_IN_STOPPED, priority_,
            &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::ecore_audio_in, function_type>,
         ECORE_AUDIO_IN_EVENT_IN_STOPPED );
   }

   /// Called when an input has stopped playing.
   template <typename T>
   void
   callback_in_stopped_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (ECORE_AUDIO_IN_EVENT_IN_STOPPED, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(ECORE_AUDIO_IN_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::ecore_audio_in::address_of<address_of>
      , ::eo_cxx::ecore_audio::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::ecore_audio_in* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::ecore_audio_in::address_const_of<address_const_of>
      , ::eo_cxx::ecore_audio::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::ecore_audio_in const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(ECORE_AUDIO_IN_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::ecore_audio_in) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::ecore_audio_in>::value, "");


#include "ecore_audio_in.eo.impl.hh"

#endif // EFL_GENERATED_ECORE_AUDIO_IN_HH

