#ifndef EFL_GENERATED_EVAS_3D_CAMERA_HH
#define EFL_GENERATED_EVAS_3D_CAMERA_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_3d_camera.eo.h"
}

#include "evas_3d_object.eo.hh"
#include "evas_common_interface.eo.hh"

struct evas_3d_camera;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct evas_3d_camera
{
   /// @brief Set the projection matrix of the given camera.
   ///
   /// @param camera     The given camera.
   /// @param matrix     Pointer to the array of 16 Evas_Real values in column major order.
   ///
   /// Default projection matrix is identity matrix.
   ///
   /// @see evas_3d_camera_projection_perspective_set()
   /// @see evas_3d_camera_projection_ortho_set()
   /// @see evas_3d_camera_projection_frustum_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param matrix Projection Matrix
   ///
   void projection_matrix_set(const Evas_Real * matrix_) const;

   /// @brief Get the projection matrix of the given camera.
   ///
   /// @param camera     The given camera.
   /// @param matrix     Pointer to receive the 16 Evas_Real values in column major order.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param[out] matrix Projection Matrix
   ///
   void projection_matrix_get(Evas_Real* matrix_) const;

   /// @brief Set the projection matrix of the given camera with perspective projection.
   ///
   /// @param camera     The given camera.
   /// @param fovy       Field of view angle in Y direction.
   /// @param aspect     Aspect ratio.
   /// @param dnear      Distance to near clipping plane.
   /// @param dfar       Distance to far clipping plane.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param fovy Field of view angle in Y direction.
   /// @param aspect Aspect ratio.
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_perspective_set(Evas_Real fovy_, Evas_Real aspect_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given camera with frustum projection.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_frustum_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given camera with orthogonal projection.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_ortho_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Check is bounding sphere of given node inside frustum of camera node.
   /// *
   /// * @param camera          The given camera node.
   /// * @param node            The given node.
   /// * @return                @c EINA_TRUE in frustum, @c EINA_FALSE otherwise
   ///
   /// * If the nodes are @ NULL or nodes type mismatch error wrong type of nodes will be generated and returned @ EINA_FALSE.
   ///
   /// * @ingroup Evas_3D_Camera
   ///
   /// @param camera_node 
   /// @param node 
   /// @param key 
   ///
   bool node_visible_get(Evas_3D_Node * camera_node_, Evas_3D_Node * node_, Evas_3D_Frustum_Mode key_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_CAMERA_CLASS);
   }

   operator ::evas_3d_camera() const;
   operator ::evas_3d_camera&();
   operator ::evas_3d_camera const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas_3d_camera*() const { return static_cast<::evas_3d_camera*>(static_cast<D const*>(this)->p); }
      operator ::evas_3d_camera const*() const { return static_cast<::evas_3d_camera const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas_3d_camera const*() const { return static_cast<::evas_3d_camera const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class evas_3d_camera
struct evas_3d_camera
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas_3d_object)
{
   //@{
   /**
      @brief Constructs a new evas_3d_camera object.

      Constructs a new evas_3d_camera object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas_3d_camera my_evas_3d_camera(efl::eo::parent = parent_object);
      @endcode

      @see evas_3d_camera(Eo* eo)
   */
   explicit evas_3d_camera(::efl::eo::parent_type _p)
      : evas_3d_camera(_ctors_call(_p))
   {}

   explicit evas_3d_camera()
      : evas_3d_camera(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit evas_3d_camera(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit evas_3d_camera(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   evas_3d_camera(evas_3d_camera const& other)
      : evas_3d_camera(eo_ref(other._eo_ptr()))
   {}

   ~evas_3d_camera() {}

   /// @brief Set the projection matrix of the given camera.
   ///
   /// @param camera     The given camera.
   /// @param matrix     Pointer to the array of 16 Evas_Real values in column major order.
   ///
   /// Default projection matrix is identity matrix.
   ///
   /// @see evas_3d_camera_projection_perspective_set()
   /// @see evas_3d_camera_projection_ortho_set()
   /// @see evas_3d_camera_projection_frustum_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param matrix Projection Matrix
   ///
   void projection_matrix_set(const Evas_Real * matrix_) const;

   /// @brief Get the projection matrix of the given camera.
   ///
   /// @param camera     The given camera.
   /// @param matrix     Pointer to receive the 16 Evas_Real values in column major order.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param[out] matrix Projection Matrix
   ///
   void projection_matrix_get(Evas_Real* matrix_) const;

   /// @brief Set the projection matrix of the given camera with perspective projection.
   ///
   /// @param camera     The given camera.
   /// @param fovy       Field of view angle in Y direction.
   /// @param aspect     Aspect ratio.
   /// @param dnear      Distance to near clipping plane.
   /// @param dfar       Distance to far clipping plane.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param fovy Field of view angle in Y direction.
   /// @param aspect Aspect ratio.
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_perspective_set(Evas_Real fovy_, Evas_Real aspect_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given camera with frustum projection.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_frustum_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given camera with orthogonal projection.
   ///
   /// @see evas_3d_camera_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Camera
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_ortho_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Check is bounding sphere of given node inside frustum of camera node.
   /// *
   /// * @param camera          The given camera node.
   /// * @param node            The given node.
   /// * @return                @c EINA_TRUE in frustum, @c EINA_FALSE otherwise
   ///
   /// * If the nodes are @ NULL or nodes type mismatch error wrong type of nodes will be generated and returned @ EINA_FALSE.
   ///
   /// * @ingroup Evas_3D_Camera
   ///
   /// @param camera_node 
   /// @param node 
   /// @param key 
   ///
   bool node_visible_get(Evas_3D_Node * camera_node_, Evas_3D_Node * node_, Evas_3D_Frustum_Mode key_) const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Clicked Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_clicked_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_CLICKED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>,
         EVAS_3D_OBJECT_EVENT_CLICKED );
   }

   /// Clicked Event
   template <typename T>
   void
   callback_clicked_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_CLICKED, info));
   }

   /// Collision Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_collision_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_COLLISION, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>,
         EVAS_3D_OBJECT_EVENT_COLLISION );
   }

   /// Collision Event
   template <typename T>
   void
   callback_collision_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_COLLISION, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_camera, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_CAMERA_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas_3d_camera::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas_3d_object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas_3d_camera* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas_3d_camera::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas_3d_object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas_3d_camera const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_3D_CAMERA_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas_3d_camera) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas_3d_camera>::value, "");


#include "evas_3d_camera.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_3D_CAMERA_HH

