#ifndef EFL_GENERATED_EVAS_3D_LIGHT_HH
#define EFL_GENERATED_EVAS_3D_LIGHT_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_3d_light.eo.h"
}

#include "evas_3d_object.eo.hh"
#include "evas_common_interface.eo.hh"

struct evas_3d_light;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct evas_3d_light
{
   /// @brief Set the ambient color of the given light.
   ///
   /// Default ambient color is (0.0, 0.0, 0.0, 1.0).
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param r Red component of the ambient color between [0.0, 1.0].
   /// @param g Green component of the ambient color between [0.0, 1.0].
   /// @param b Blue component of the ambient color between [0.0, 1.0].
   /// @param a Alpha component of the ambient color between [0.0, 1.0].
   ///
   void ambient_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @brief Get the ambient color of the given light.
   ///
   /// @see evas_3d_light_ambient_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] r Red component of the ambient color between [0.0, 1.0].
   /// @param[out] g Green component of the ambient color between [0.0, 1.0].
   /// @param[out] b Blue component of the ambient color between [0.0, 1.0].
   /// @param[out] a Alpha component of the ambient color between [0.0, 1.0].
   ///
   void ambient_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @brief Set the diffuse color of the given light.
   ///
   /// Default diffuse color is (1.0, 1.0, 1.0, 1.0).
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param r Red component of the diffuse color between [0.0, 1.0].
   /// @param g Green component of the diffuse color between [0.0, 1.0].
   /// @param b Blue component of the diffuse color between [0.0, 1.0].
   /// @param a Alpha component of the diffuse color between [0.0, 1.0].
   ///
   void diffuse_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @brief Get the diffuse color of the given light.
   ///
   /// @see evas_3d_light_diffuse_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] r Red component of the diffuse color between [0.0, 1.0].
   /// @param[out] g Green component of the diffuse color between [0.0, 1.0].
   /// @param[out] b Blue component of the diffuse color between [0.0, 1.0].
   /// @param[out] a Alpha component of the diffuse color between [0.0, 1.0].
   ///
   void diffuse_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @brief Set the specular color of the given light.
   ///
   /// Default specular color is (1.0, 1.0, 1.0, 1.0).
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param r Red component of the specular color between [0.0, 1.0].
   /// @param g Green component of the specular color between [0.0, 1.0].
   /// @param b Blue component of the specular color between [0.0, 1.0].
   /// @param a Alpha component of the specular color between [0.0, 1.0].
   ///
   void specular_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @brief Get the specular color of the given light.
   ///
   /// @see evas_3d_light_specular_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] r Red component of the specular color between [0.0, 1.0].
   /// @param[out] g Green component of the specular color between [0.0, 1.0].
   /// @param[out] b Blue component of the specular color between [0.0, 1.0].
   /// @param[out] a Alpha component of the specular color between [0.0, 1.0].
   ///
   void specular_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @brief Set the attenuation of the given light.
   ///
   /// Light attenuation has no effect with directional light. And the attenuation
   /// should be enabled first to take effect. The attenuation factor is calculated
   /// as follows.
   ///
   /// atten = 1.0 / constant + linear * distance + quadratic * distance * distance
   ///
   /// Default attenuation is constant = 1.0, linear = 0.0, quadratic = 0.0.
   ///
   /// @see evas_3d_light_attenuation_enable_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param constant Constant attenuation term..
   /// @param linear Linear attenuation term..
   /// @param quadratic Quadratic attenuation term..
   ///
   void attenuation_set(Evas_Real constant_, Evas_Real linear_, Evas_Real quadratic_) const;

   /// @brief Get the attenuation of the given light.
   ///
   /// @see evas_3d_light_attenuation_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] constant Constant attenuation term..
   /// @param[out] linear Linear attenuation term..
   /// @param[out] quadratic Quadratic attenuation term..
   ///
   void attenuation_get(Evas_Real* constant_, Evas_Real* linear_, Evas_Real* quadratic_) const;

   /// @brief Set the projection matrix of the given light source.
   ///
   /// @param light The given light source.
   /// @param matrix Pointer to the array of 16 Evas_Real values in column major order.
   ///
   /// Default projection matrix is identity matrix.
   ///
   /// @see evas_3d_light_projection_perspective_set()
   /// @see evas_3d_light_projection_ortho_set()
   /// @see evas_3d_light_projection_frustum_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param matrix Projection Matrix
   ///
   void projection_matrix_set(const Evas_Real * matrix_) const;

   /// @brief Get the projection matrix of the given light source.
   ///
   /// @param light The given light source.
   /// @param matrix Pointer to receive the 16 Evas_Real values in column major order.
   ///
   /// @see evas_3d_light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] matrix Projection Matrix
   ///
   void projection_matrix_get(Evas_Real* matrix_) const;

   /// @brief Set the projection matrix of the given light source with perspective projection.
   ///
   /// @param light The given light source.
   /// @param fovy Field of view angle in Y direction.
   /// @param aspect Aspect ratio.
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   /// @see evas_3d_light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param fovy Field of view angle in Y direction.
   /// @param aspect Aspect ratio.
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_perspective_set(Evas_Real fovy_, Evas_Real aspect_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given light source with frustum projection.
   ///
   /// @see evas_3d_Light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_frustum_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given light source with orthogonal projection.
   ///
   /// @see evas_3d_light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_ortho_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Get the directional flag of the given light.
   ///
   /// @see evas_3d_light_directional_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param directional Whether the light is directional (@c EINA_TRUE), or not (@c EINA_FALSE).
   ///
   bool directional_get() const;

   /// @brief Set the directional flag of the given light.
   ///
   /// Directional light is a type of light which is infinitely far away with no
   /// attenuation. The light direction is determined by the containing node's
   /// forward vector (negative Z-axis).
   ///
   /// By default, directional is not enabled.
   ///
   /// @see evas_3d_node_look_at_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param directional Whether the light is directional (@c EINA_TRUE), or not (@c EINA_FALSE).
   ///
   void directional_set(bool directional_) const;

   /// @brief Get the spot exponent of the given light.
   ///
   /// @see evas_3d_light_spot_exponent_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param exponent Spot exponent value..
   ///
   Evas_Real spot_exponent_get() const;

   /// @brief Set the spot exponent of the given light.
   ///
   /// Higher spot exponent means intensity at the center of the cone is relatively
   /// stronger. Zero exponent means the light intensity is evenly distibuted. The
   /// spot exponent has no effect when the light is not spot light (spot cutoff
   /// angle is less than 180 degree).
   ///
   /// Default spot exponent is 0.
   ///
   /// @see evas_3d_light_spot_cutoff_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param exponent Spot exponent value..
   ///
   void spot_exponent_set(Evas_Real exponent_) const;

   /// @brief Get the spot cutoff angle of the given light.
   ///
   /// @see evas_3d_light_spot_cutoff_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param cutoff Cutoff angle in degree..
   ///
   Evas_Real spot_cutoff_get() const;

   /// @brief Set the spot cutoff angle of the given light.
   ///
   /// Only angle less than 180 degree will make it spot light, so that other spot
   /// light attribute will take effect.
   ///
   /// Default spot cutoff angle is 180.
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param cutoff Cutoff angle in degree..
   ///
   void spot_cutoff_set(Evas_Real cutoff_) const;

   /// @brief Get the attenuation enable flag of the given light.
   ///
   /// @see evas_3d_light_attenuation_enable_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param enable Whether to enable attenuation (@c EINA_TRUE), or not (@c EINA_FALSE)..
   ///
   bool attenuation_enable_get() const;

   /// @brief Set the attenuation enable flag of the given light.
   ///
   /// By default, light attenuation is not enabled.
   ///
   /// @see evas_3d_light_attenuation_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param enable Whether to enable attenuation (@c EINA_TRUE), or not (@c EINA_FALSE)..
   ///
   void attenuation_enable_set(bool enable_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_LIGHT_CLASS);
   }

   operator ::evas_3d_light() const;
   operator ::evas_3d_light&();
   operator ::evas_3d_light const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas_3d_light*() const { return static_cast<::evas_3d_light*>(static_cast<D const*>(this)->p); }
      operator ::evas_3d_light const*() const { return static_cast<::evas_3d_light const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas_3d_light const*() const { return static_cast<::evas_3d_light const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class evas_3d_light
struct evas_3d_light
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas_3d_object)
{
   //@{
   /**
      @brief Constructs a new evas_3d_light object.

      Constructs a new evas_3d_light object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas_3d_light my_evas_3d_light(efl::eo::parent = parent_object);
      @endcode

      @see evas_3d_light(Eo* eo)
   */
   explicit evas_3d_light(::efl::eo::parent_type _p)
      : evas_3d_light(_ctors_call(_p))
   {}

   explicit evas_3d_light()
      : evas_3d_light(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit evas_3d_light(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit evas_3d_light(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   evas_3d_light(evas_3d_light const& other)
      : evas_3d_light(eo_ref(other._eo_ptr()))
   {}

   ~evas_3d_light() {}

   /// @brief Set the ambient color of the given light.
   ///
   /// Default ambient color is (0.0, 0.0, 0.0, 1.0).
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param r Red component of the ambient color between [0.0, 1.0].
   /// @param g Green component of the ambient color between [0.0, 1.0].
   /// @param b Blue component of the ambient color between [0.0, 1.0].
   /// @param a Alpha component of the ambient color between [0.0, 1.0].
   ///
   void ambient_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @brief Get the ambient color of the given light.
   ///
   /// @see evas_3d_light_ambient_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] r Red component of the ambient color between [0.0, 1.0].
   /// @param[out] g Green component of the ambient color between [0.0, 1.0].
   /// @param[out] b Blue component of the ambient color between [0.0, 1.0].
   /// @param[out] a Alpha component of the ambient color between [0.0, 1.0].
   ///
   void ambient_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @brief Set the diffuse color of the given light.
   ///
   /// Default diffuse color is (1.0, 1.0, 1.0, 1.0).
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param r Red component of the diffuse color between [0.0, 1.0].
   /// @param g Green component of the diffuse color between [0.0, 1.0].
   /// @param b Blue component of the diffuse color between [0.0, 1.0].
   /// @param a Alpha component of the diffuse color between [0.0, 1.0].
   ///
   void diffuse_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @brief Get the diffuse color of the given light.
   ///
   /// @see evas_3d_light_diffuse_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] r Red component of the diffuse color between [0.0, 1.0].
   /// @param[out] g Green component of the diffuse color between [0.0, 1.0].
   /// @param[out] b Blue component of the diffuse color between [0.0, 1.0].
   /// @param[out] a Alpha component of the diffuse color between [0.0, 1.0].
   ///
   void diffuse_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @brief Set the specular color of the given light.
   ///
   /// Default specular color is (1.0, 1.0, 1.0, 1.0).
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param r Red component of the specular color between [0.0, 1.0].
   /// @param g Green component of the specular color between [0.0, 1.0].
   /// @param b Blue component of the specular color between [0.0, 1.0].
   /// @param a Alpha component of the specular color between [0.0, 1.0].
   ///
   void specular_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @brief Get the specular color of the given light.
   ///
   /// @see evas_3d_light_specular_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] r Red component of the specular color between [0.0, 1.0].
   /// @param[out] g Green component of the specular color between [0.0, 1.0].
   /// @param[out] b Blue component of the specular color between [0.0, 1.0].
   /// @param[out] a Alpha component of the specular color between [0.0, 1.0].
   ///
   void specular_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @brief Set the attenuation of the given light.
   ///
   /// Light attenuation has no effect with directional light. And the attenuation
   /// should be enabled first to take effect. The attenuation factor is calculated
   /// as follows.
   ///
   /// atten = 1.0 / constant + linear * distance + quadratic * distance * distance
   ///
   /// Default attenuation is constant = 1.0, linear = 0.0, quadratic = 0.0.
   ///
   /// @see evas_3d_light_attenuation_enable_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param constant Constant attenuation term..
   /// @param linear Linear attenuation term..
   /// @param quadratic Quadratic attenuation term..
   ///
   void attenuation_set(Evas_Real constant_, Evas_Real linear_, Evas_Real quadratic_) const;

   /// @brief Get the attenuation of the given light.
   ///
   /// @see evas_3d_light_attenuation_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] constant Constant attenuation term..
   /// @param[out] linear Linear attenuation term..
   /// @param[out] quadratic Quadratic attenuation term..
   ///
   void attenuation_get(Evas_Real* constant_, Evas_Real* linear_, Evas_Real* quadratic_) const;

   /// @brief Set the projection matrix of the given light source.
   ///
   /// @param light The given light source.
   /// @param matrix Pointer to the array of 16 Evas_Real values in column major order.
   ///
   /// Default projection matrix is identity matrix.
   ///
   /// @see evas_3d_light_projection_perspective_set()
   /// @see evas_3d_light_projection_ortho_set()
   /// @see evas_3d_light_projection_frustum_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param matrix Projection Matrix
   ///
   void projection_matrix_set(const Evas_Real * matrix_) const;

   /// @brief Get the projection matrix of the given light source.
   ///
   /// @param light The given light source.
   /// @param matrix Pointer to receive the 16 Evas_Real values in column major order.
   ///
   /// @see evas_3d_light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param[out] matrix Projection Matrix
   ///
   void projection_matrix_get(Evas_Real* matrix_) const;

   /// @brief Set the projection matrix of the given light source with perspective projection.
   ///
   /// @param light The given light source.
   /// @param fovy Field of view angle in Y direction.
   /// @param aspect Aspect ratio.
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   /// @see evas_3d_light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param fovy Field of view angle in Y direction.
   /// @param aspect Aspect ratio.
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_perspective_set(Evas_Real fovy_, Evas_Real aspect_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given light source with frustum projection.
   ///
   /// @see evas_3d_Light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_frustum_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Set the projection matrix of the given light source with orthogonal projection.
   ///
   /// @see evas_3d_light_projection_matrix_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param left Left X coordinate of the near clipping plane.
   /// @param right Right X coordinate of the near clipping plane..
   /// @param bottom Bottom Y coordinate of the near clipping plane.
   /// @param top Top Y coordinate of the near clipping plane
   /// @param dnear Distance to near clipping plane.
   /// @param dfar Distance to far clipping plane.
   ///
   void projection_ortho_set(Evas_Real left_, Evas_Real right_, Evas_Real bottom_, Evas_Real top_, Evas_Real dnear_, Evas_Real dfar_) const;

   /// @brief Get the directional flag of the given light.
   ///
   /// @see evas_3d_light_directional_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param directional Whether the light is directional (@c EINA_TRUE), or not (@c EINA_FALSE).
   ///
   bool directional_get() const;

   /// @brief Set the directional flag of the given light.
   ///
   /// Directional light is a type of light which is infinitely far away with no
   /// attenuation. The light direction is determined by the containing node's
   /// forward vector (negative Z-axis).
   ///
   /// By default, directional is not enabled.
   ///
   /// @see evas_3d_node_look_at_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param directional Whether the light is directional (@c EINA_TRUE), or not (@c EINA_FALSE).
   ///
   void directional_set(bool directional_) const;

   /// @brief Get the spot exponent of the given light.
   ///
   /// @see evas_3d_light_spot_exponent_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param exponent Spot exponent value..
   ///
   Evas_Real spot_exponent_get() const;

   /// @brief Set the spot exponent of the given light.
   ///
   /// Higher spot exponent means intensity at the center of the cone is relatively
   /// stronger. Zero exponent means the light intensity is evenly distibuted. The
   /// spot exponent has no effect when the light is not spot light (spot cutoff
   /// angle is less than 180 degree).
   ///
   /// Default spot exponent is 0.
   ///
   /// @see evas_3d_light_spot_cutoff_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param exponent Spot exponent value..
   ///
   void spot_exponent_set(Evas_Real exponent_) const;

   /// @brief Get the spot cutoff angle of the given light.
   ///
   /// @see evas_3d_light_spot_cutoff_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param cutoff Cutoff angle in degree..
   ///
   Evas_Real spot_cutoff_get() const;

   /// @brief Set the spot cutoff angle of the given light.
   ///
   /// Only angle less than 180 degree will make it spot light, so that other spot
   /// light attribute will take effect.
   ///
   /// Default spot cutoff angle is 180.
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param cutoff Cutoff angle in degree..
   ///
   void spot_cutoff_set(Evas_Real cutoff_) const;

   /// @brief Get the attenuation enable flag of the given light.
   ///
   /// @see evas_3d_light_attenuation_enable_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param enable Whether to enable attenuation (@c EINA_TRUE), or not (@c EINA_FALSE)..
   ///
   bool attenuation_enable_get() const;

   /// @brief Set the attenuation enable flag of the given light.
   ///
   /// By default, light attenuation is not enabled.
   ///
   /// @see evas_3d_light_attenuation_set()
   ///
   /// @ingroup Evas_3D_Light
   ///
   /// @param enable Whether to enable attenuation (@c EINA_TRUE), or not (@c EINA_FALSE)..
   ///
   void attenuation_enable_set(bool enable_) const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Clicked Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_clicked_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_CLICKED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>,
         EVAS_3D_OBJECT_EVENT_CLICKED );
   }

   /// Clicked Event
   template <typename T>
   void
   callback_clicked_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_CLICKED, info));
   }

   /// Collision Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_collision_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_COLLISION, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>,
         EVAS_3D_OBJECT_EVENT_COLLISION );
   }

   /// Collision Event
   template <typename T>
   void
   callback_collision_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_COLLISION, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_light, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_LIGHT_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas_3d_light::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas_3d_object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas_3d_light* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas_3d_light::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas_3d_object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas_3d_light const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_3D_LIGHT_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas_3d_light) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas_3d_light>::value, "");


#include "evas_3d_light.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_3D_LIGHT_HH

