#ifndef EFL_GENERATED_EVAS_3D_MESH_HH
#define EFL_GENERATED_EVAS_3D_MESH_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_3d_mesh.eo.h"
}

#include "evas_3d_object.eo.hh"
#include "evas_common_interface.eo.hh"
#include "efl_file.eo.hh"

struct evas_3d_mesh;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct evas_3d_mesh
{
   /// @brief Set the vertex data of the key frame of the given mesh.
   ///
   /// This function make evas read from the given buffer whenever it requires.
   /// If you want to release the buffer after calling this functions, use
   /// evas_3d_mesh_frame_vertex_data_copy_set() instead.
   ///
   /// After setting the vertex data, further modifications should be protected
   /// by map/unmap pair.
   ///
   /// @see evas_3d_mesh_frame_vertex_data_copy_set()
   /// @see evas_3d_mesh_frame_vertex_data_map()
   /// @see evas_3d_mesh_frame_vertex_data_unmap()
   ///
   /// @ingroup Evas_3D_Mesh
   ///
   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   /// @param stride Stride to go to the next vertex (in bytes).
   /// @param data Pointer to the vertex data buffer.
   ///
   void frame_vertex_data_set(int frame_, Evas_3D_Vertex_Attrib attrib_, int stride_, const void * data_) const;

   /// @brief Set the vertex data of the key frame of the given mesh by copying from a buffer.
   /// This function allocates internal vertex buffer and copy from the given
   /// buffer. So you can release the buffer. If you want to modify the vertex data
   /// use evas_3d_mesh_frame_vertex_data_map(). After finishing the modifications,
   /// you should call evas_3d_mesh_frame_vertex_data_unmap().
   ///
   /// @see evas_3d_mesh_frame_vertex_data_set()
   /// @see evas_3d_mesh_frame_vertex_data_map()
   /// @see evas_3d_mesh_frame_vertex_data_unmap()
   ///
   /// @ingroup Evas_3D_Mesh
   ///
   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   /// @param stride Stride to go to the next vertex (in bytes).
   /// @param data Pointer to the vertex data buffer.
   ///
   void frame_vertex_data_copy_set(int frame_, Evas_3D_Vertex_Attrib attrib_, int stride_, const void * data_) const;

   /// @brief Map the vertex buffer of the key frame of the given mesh.
   ///
   /// After manipulating the mapped buffer, evas_3d_mesh_frame_vertex_data_unmap()
   /// should be called to properly download the data to the engine. If the data
   /// was set using evas_3d_mesh_frame_vertex_data_set(), pointer to the original
   /// buffer will be returned. Otherwise, the returned pointer can differ every
   /// time calling this function.
   ///
   /// @see evas_3d_mesh_frame_vertex_data_unmap()
   /// @return Starting address of the mapped vertex buffer.
   ///
   /// @ingroup Evas_3D_Mesh
   ///
   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   ///
   void * frame_vertex_data_map(int frame_, Evas_3D_Vertex_Attrib attrib_) const;

   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   ///
   void frame_vertex_data_unmap(int frame_, Evas_3D_Vertex_Attrib attrib_) const;

   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   ///
   int frame_vertex_stride_get(int frame_, Evas_3D_Vertex_Attrib attrib_) const;

   /// @param format Vertex index data format.
   /// @param count Vertex index count.
   /// @param indices Pointer to the index data.
   ///
   void index_data_set(Evas_3D_Index_Format format_, int count_, const void * indices_) const;

   /// @param format Vertex index data format.
   /// @param count Vertex index count.
   /// @param indices Pointer to the index data.
   ///
   void index_data_copy_set(Evas_3D_Index_Format format_, int count_, const void * indices_) const;

   Evas_3D_Index_Format index_format_get() const;

   int index_count_get() const;

   void * index_data_map() const;

   void index_data_unmap() const;

   /// @param frame The number of the key frame to be added.
   ///
   void frame_add(int frame_) const;

   /// @param frame The number of the key frame to be added.
   ///
   void frame_del(int frame_) const;

   /// @param r The red component of the fog color.
   /// @param g The green component of the fog color.
   /// @param b The blue component of the fog color.
   /// @param a The transparency of fog.
   ///
   void fog_color_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @param[out] r Pointer to receive red component of the fog color.
   /// @param[out] g Pointer to receive green component of the fog color.
   /// @param[out] b Pointer to receive blue component of the fog color.
   /// @param[out] a Pointer to receive transparency of fog.
   ///
   void fog_color_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @param sfactor Specifies how the red, green, blue, and alpha source blending factors are computed.
   /// @param dfactor Specifies how the red, green, blue, and alpha destination blending factors are computed.
   ///
   void blending_func_set(Evas_3D_Blend_Func sfactor_, Evas_3D_Blend_Func dfactor_) const;

   /// @param[out] sfactor Pointer to receive source blendin factors key.
   /// @param[out] dfactor Pointer to receive destination blendin factors key.
   ///
   void blending_func_get(Evas_3D_Blend_Func* sfactor_, Evas_3D_Blend_Func* dfactor_) const;

   /// @param mode The shade mode.
   ///
   Evas_3D_Shade_Mode shade_mode_get() const;

   /// @param mode The shade mode.
   ///
   void shade_mode_set(Evas_3D_Shade_Mode mode_) const;

   /// @param enabled Posibility flag
   ///
   bool color_pick_enable_get() const;

   /// @param enabled Posibility flag
   ///
   void color_pick_enable_set(bool enabled_) const;

   /// @param count Vertex count.
   ///
   unsigned int vertex_count_get() const;

   /// @param count Vertex count.
   ///
   void vertex_count_set(unsigned int count_) const;

   /// @param enabled Fog enabled status.
   ///
   bool fog_enable_get() const;

   /// @param enabled Fog enabled status.
   ///
   void fog_enable_set(bool enabled_) const;

   /// @param enabled Alpha test status.
   ///
   bool alpha_test_enable_get() const;

   /// @param enabled Alpha test status.
   ///
   void alpha_test_enable_set(bool enabled_) const;

   /// @param comparison_func Specifies the alpha comparison function.
   /// @param ref_value Specifies the reference value that incoming alpha values are compared to.
   ///
   void alpha_func_get(Evas_3D_Comparison* comparison_func_, Evas_Real* ref_value_) const;

   /// @param comparison_func Specifies the alpha comparison function.
   /// @param ref_value Specifies the reference value that incoming alpha values are compared to.
   ///
   void alpha_func_set(Evas_3D_Comparison comparison_func_, Evas_Real ref_value_) const;

   /// @param blending The blending state.
   ///
   bool blending_enable_get() const;

   /// @param blending The blending state.
   ///
   void blending_enable_set(bool blending_) const;

   /// @param material The material to be set to the key frame.
   ///
   Evas_3D_Material * frame_material_get(int frame_) const;

   /// @param material The material to be set to the key frame.
   ///
   void frame_material_set(int frame_, Evas_3D_Material * material_) const;

   /// @param assembly Vertex assembly.
   ///
   Evas_3D_Vertex_Assembly vertex_assembly_get() const;

   /// @param assembly Vertex assembly.
   ///
   void vertex_assembly_set(Evas_3D_Vertex_Assembly assembly_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_MESH_CLASS);
   }

   operator ::evas_3d_mesh() const;
   operator ::evas_3d_mesh&();
   operator ::evas_3d_mesh const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas_3d_mesh*() const { return static_cast<::evas_3d_mesh*>(static_cast<D const*>(this)->p); }
      operator ::evas_3d_mesh const*() const { return static_cast<::evas_3d_mesh const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas_3d_mesh const*() const { return static_cast<::evas_3d_mesh const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class evas_3d_mesh
struct evas_3d_mesh
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(efl::file)
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas_3d_object)
{
   //@{
   /**
      @brief Constructs a new evas_3d_mesh object.

      Constructs a new evas_3d_mesh object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas_3d_mesh my_evas_3d_mesh(efl::eo::parent = parent_object);
      @endcode

      @see evas_3d_mesh(Eo* eo)
   */
   explicit evas_3d_mesh(::efl::eo::parent_type _p)
      : evas_3d_mesh(_ctors_call(_p))
   {}

   explicit evas_3d_mesh()
      : evas_3d_mesh(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit evas_3d_mesh(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit evas_3d_mesh(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   evas_3d_mesh(evas_3d_mesh const& other)
      : evas_3d_mesh(eo_ref(other._eo_ptr()))
   {}

   ~evas_3d_mesh() {}

   /// @brief Set the vertex data of the key frame of the given mesh.
   ///
   /// This function make evas read from the given buffer whenever it requires.
   /// If you want to release the buffer after calling this functions, use
   /// evas_3d_mesh_frame_vertex_data_copy_set() instead.
   ///
   /// After setting the vertex data, further modifications should be protected
   /// by map/unmap pair.
   ///
   /// @see evas_3d_mesh_frame_vertex_data_copy_set()
   /// @see evas_3d_mesh_frame_vertex_data_map()
   /// @see evas_3d_mesh_frame_vertex_data_unmap()
   ///
   /// @ingroup Evas_3D_Mesh
   ///
   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   /// @param stride Stride to go to the next vertex (in bytes).
   /// @param data Pointer to the vertex data buffer.
   ///
   void frame_vertex_data_set(int frame_, Evas_3D_Vertex_Attrib attrib_, int stride_, const void * data_) const;

   /// @brief Set the vertex data of the key frame of the given mesh by copying from a buffer.
   /// This function allocates internal vertex buffer and copy from the given
   /// buffer. So you can release the buffer. If you want to modify the vertex data
   /// use evas_3d_mesh_frame_vertex_data_map(). After finishing the modifications,
   /// you should call evas_3d_mesh_frame_vertex_data_unmap().
   ///
   /// @see evas_3d_mesh_frame_vertex_data_set()
   /// @see evas_3d_mesh_frame_vertex_data_map()
   /// @see evas_3d_mesh_frame_vertex_data_unmap()
   ///
   /// @ingroup Evas_3D_Mesh
   ///
   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   /// @param stride Stride to go to the next vertex (in bytes).
   /// @param data Pointer to the vertex data buffer.
   ///
   void frame_vertex_data_copy_set(int frame_, Evas_3D_Vertex_Attrib attrib_, int stride_, const void * data_) const;

   /// @brief Map the vertex buffer of the key frame of the given mesh.
   ///
   /// After manipulating the mapped buffer, evas_3d_mesh_frame_vertex_data_unmap()
   /// should be called to properly download the data to the engine. If the data
   /// was set using evas_3d_mesh_frame_vertex_data_set(), pointer to the original
   /// buffer will be returned. Otherwise, the returned pointer can differ every
   /// time calling this function.
   ///
   /// @see evas_3d_mesh_frame_vertex_data_unmap()
   /// @return Starting address of the mapped vertex buffer.
   ///
   /// @ingroup Evas_3D_Mesh
   ///
   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   ///
   void * frame_vertex_data_map(int frame_, Evas_3D_Vertex_Attrib attrib_) const;

   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   ///
   void frame_vertex_data_unmap(int frame_, Evas_3D_Vertex_Attrib attrib_) const;

   /// @param frame The number of the key frame.
   /// @param attrib Vertex attribute ID.
   ///
   int frame_vertex_stride_get(int frame_, Evas_3D_Vertex_Attrib attrib_) const;

   /// @param format Vertex index data format.
   /// @param count Vertex index count.
   /// @param indices Pointer to the index data.
   ///
   void index_data_set(Evas_3D_Index_Format format_, int count_, const void * indices_) const;

   /// @param format Vertex index data format.
   /// @param count Vertex index count.
   /// @param indices Pointer to the index data.
   ///
   void index_data_copy_set(Evas_3D_Index_Format format_, int count_, const void * indices_) const;

   Evas_3D_Index_Format index_format_get() const;

   int index_count_get() const;

   void * index_data_map() const;

   void index_data_unmap() const;

   /// @param frame The number of the key frame to be added.
   ///
   void frame_add(int frame_) const;

   /// @param frame The number of the key frame to be added.
   ///
   void frame_del(int frame_) const;

   /// @param r The red component of the fog color.
   /// @param g The green component of the fog color.
   /// @param b The blue component of the fog color.
   /// @param a The transparency of fog.
   ///
   void fog_color_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @param[out] r Pointer to receive red component of the fog color.
   /// @param[out] g Pointer to receive green component of the fog color.
   /// @param[out] b Pointer to receive blue component of the fog color.
   /// @param[out] a Pointer to receive transparency of fog.
   ///
   void fog_color_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @param sfactor Specifies how the red, green, blue, and alpha source blending factors are computed.
   /// @param dfactor Specifies how the red, green, blue, and alpha destination blending factors are computed.
   ///
   void blending_func_set(Evas_3D_Blend_Func sfactor_, Evas_3D_Blend_Func dfactor_) const;

   /// @param[out] sfactor Pointer to receive source blendin factors key.
   /// @param[out] dfactor Pointer to receive destination blendin factors key.
   ///
   void blending_func_get(Evas_3D_Blend_Func* sfactor_, Evas_3D_Blend_Func* dfactor_) const;

   /// @param mode The shade mode.
   ///
   Evas_3D_Shade_Mode shade_mode_get() const;

   /// @param mode The shade mode.
   ///
   void shade_mode_set(Evas_3D_Shade_Mode mode_) const;

   /// @param enabled Posibility flag
   ///
   bool color_pick_enable_get() const;

   /// @param enabled Posibility flag
   ///
   void color_pick_enable_set(bool enabled_) const;

   /// @param count Vertex count.
   ///
   unsigned int vertex_count_get() const;

   /// @param count Vertex count.
   ///
   void vertex_count_set(unsigned int count_) const;

   /// @param enabled Fog enabled status.
   ///
   bool fog_enable_get() const;

   /// @param enabled Fog enabled status.
   ///
   void fog_enable_set(bool enabled_) const;

   /// @param enabled Alpha test status.
   ///
   bool alpha_test_enable_get() const;

   /// @param enabled Alpha test status.
   ///
   void alpha_test_enable_set(bool enabled_) const;

   /// @param comparison_func Specifies the alpha comparison function.
   /// @param ref_value Specifies the reference value that incoming alpha values are compared to.
   ///
   void alpha_func_get(Evas_3D_Comparison* comparison_func_, Evas_Real* ref_value_) const;

   /// @param comparison_func Specifies the alpha comparison function.
   /// @param ref_value Specifies the reference value that incoming alpha values are compared to.
   ///
   void alpha_func_set(Evas_3D_Comparison comparison_func_, Evas_Real ref_value_) const;

   /// @param blending The blending state.
   ///
   bool blending_enable_get() const;

   /// @param blending The blending state.
   ///
   void blending_enable_set(bool blending_) const;

   /// @param material The material to be set to the key frame.
   ///
   Evas_3D_Material * frame_material_get(int frame_) const;

   /// @param material The material to be set to the key frame.
   ///
   void frame_material_set(int frame_, Evas_3D_Material * material_) const;

   /// @param assembly Vertex assembly.
   ///
   Evas_3D_Vertex_Assembly vertex_assembly_get() const;

   /// @param assembly Vertex assembly.
   ///
   void vertex_assembly_set(Evas_3D_Vertex_Assembly assembly_) const;

   /// Error occured in asynchronous file operation
   template <typename F>
   ::efl::eo::signal_connection
   callback_async_error_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EFL_FILE_EVENT_ASYNC_ERROR, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EFL_FILE_EVENT_ASYNC_ERROR );
   }

   /// Error occured in asynchronous file operation
   template <typename T>
   void
   callback_async_error_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EFL_FILE_EVENT_ASYNC_ERROR, info));
   }

   /// The file was successfully opened asynchronously
   template <typename F>
   ::efl::eo::signal_connection
   callback_async_opened_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EFL_FILE_EVENT_ASYNC_OPENED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EFL_FILE_EVENT_ASYNC_OPENED );
   }

   /// The file was successfully opened asynchronously
   template <typename T>
   void
   callback_async_opened_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EFL_FILE_EVENT_ASYNC_OPENED, info));
   }

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Clicked Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_clicked_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_CLICKED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EVAS_3D_OBJECT_EVENT_CLICKED );
   }

   /// Clicked Event
   template <typename T>
   void
   callback_clicked_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_CLICKED, info));
   }

   /// Collision Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_collision_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_COLLISION, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EVAS_3D_OBJECT_EVENT_COLLISION );
   }

   /// Collision Event
   template <typename T>
   void
   callback_collision_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_COLLISION, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_mesh, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_MESH_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas_3d_mesh::address_of<address_of>
      , ::eo_cxx::efl::file::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas_3d_object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas_3d_mesh* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas_3d_mesh::address_const_of<address_const_of>
      , ::eo_cxx::efl::file::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas_3d_object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas_3d_mesh const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_3D_MESH_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas_3d_mesh) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas_3d_mesh>::value, "");


#include "evas_3d_mesh.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_3D_MESH_HH

