#ifndef EFL_GENERATED_EVAS_3D_NODE_HH
#define EFL_GENERATED_EVAS_3D_NODE_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_3d_node.eo.h"
}

#include "evas_3d_object.eo.hh"
#include "evas_common_interface.eo.hh"
#include <eo_concrete.hh>

struct evas_3d_node;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct evas_3d_node
{
   Evas_3D_Node_Type type_get() const;

   /// @param member Node object to be added.
   ///
   void member_add(Evas_3D_Node * member_) const;

   /// @param member Member node to be deleted from the given node.
   ///
   void member_del(Evas_3D_Node * member_) const;

   Evas_3D_Node * parent_get() const;

   ::efl::eina::crange_list< Evas_3D_Node * > member_list_get() const;

   /// @param x X coordinate of the position.
   /// @param y Y coordinate of the position.
   /// @param z Z coordinate of the position.
   ///
   void position_set(Evas_Real x_, Evas_Real y_, Evas_Real z_) const;

   /// @param x X term of the orientation quaternion (w, x, y, z)
   /// @param y Y term of the orientation quaternion (w, x, y, z)
   /// @param z Z term of the orientation quaternion (w, x, y, z)
   /// @param w W term of the orientation quaternion (w, x, y, z)
   ///
   void orientation_set(Evas_Real x_, Evas_Real y_, Evas_Real z_, Evas_Real w_) const;

   /// @param angle Rotation angle.
   /// @param x X term of the rotation axis.
   /// @param y Y term of the rotation axis.
   /// @param z Z term of the rotation axis.
   ///
   void orientation_angle_axis_set(Evas_Real angle_, Evas_Real x_, Evas_Real y_, Evas_Real z_) const;

   /// @param x Scale factor along X-axis.
   /// @param y Scale factor along Y-axis.
   /// @param z Scale factor along Z-axis.
   ///
   void scale_set(Evas_Real x_, Evas_Real y_, Evas_Real z_) const;

   /// @param space 
   /// @param[out] x Pointer to receive X coordinate of the position.
   /// @param[out] y Pointer to receive Y coordinate of the position.
   /// @param[out] z Pointer to receive Z coordinate of the position.
   ///
   void position_get(Evas_3D_Space space_, Evas_Real* x_, Evas_Real* y_, Evas_Real* z_) const;

   /// @param space 
   /// @param[out] x Pointer to receive X term of the orientation quaternion.
   /// @param[out] y Pointer to receive Y term of the orientation quaternion.
   /// @param[out] z Pointer to receive Z term of the orientation quaternion.
   /// @param[out] w Pointer to receive W term of the orientation quaternion.
   ///
   void orientation_get(Evas_3D_Space space_, Evas_Real* x_, Evas_Real* y_, Evas_Real* z_, Evas_Real* w_) const;

   /// @param space 
   /// @param[out] x Pointer to receive Scale factor along X-axis.
   /// @param[out] y Pointer to receive Scale factor along X-axis.
   /// @param[out] z Pointer to receive Scale factor along X-axis.
   ///
   void scale_get(Evas_3D_Space space_, Evas_Real* x_, Evas_Real* y_, Evas_Real* z_) const;

   /// @param target_space Space where the target position belongs to.
   /// @param x X coordinate of the target position.
   /// @param y Y coordinate of the target position.
   /// @param z Z coordinate of the target position.
   /// @param up_space Space where the up vector belongs to.
   /// @param ux X term of the up vector.
   /// @param uy Y term of the up vector.
   /// @param uz Z term of the up vector.
   ///
   void look_at_set(Evas_3D_Space target_space_, Evas_Real x_, Evas_Real y_, Evas_Real z_, Evas_3D_Space up_space_, Evas_Real ux_, Evas_Real uy_, Evas_Real uz_) const;

   /// @param mesh The mesh to be added.
   ///
   void mesh_add(Evas_3D_Mesh * mesh_) const;

   /// @param mesh The mesh to be deleted.
   ///
   void mesh_del(Evas_3D_Mesh * mesh_) const;

   ::efl::eina::crange_list< Evas_3D_Mesh * > mesh_list_get() const;

   /// @param x Coordinates of vector.
   /// @param y 
   /// @param z 
   /// @param x2 
   /// @param y2 
   /// @param z2 
   ///
   void bounding_box_get(Evas_Real * x_, Evas_Real * y_, Evas_Real * z_, Evas_Real * x2_, Evas_Real * y2_, Evas_Real * z2_) const;

   /// @param x Coordinates of vector.
   /// @param y 
   /// @param z 
   /// @param r 
   ///
   void bounding_sphere_get(Evas_Real * x_, Evas_Real * y_, Evas_Real * z_, Evas_Real * r_) const;

   /// @param inherit Whether to inherit parent position
   ///
   bool position_inherit_get() const;

   /// @param inherit Whether to inherit parent position
   ///
   void position_inherit_set(bool inherit_) const;

   /// @param inherit Whether to inherit parent orientation
   ///
   bool orientation_inherit_get() const;

   /// @param inherit Whether to inherit parent orientation
   ///
   void orientation_inherit_set(bool inherit_) const;

   /// @param inherit Whether to inherit parent scale
   ///
   bool scale_inherit_get() const;

   /// @param inherit Whether to inherit parent scale
   ///
   void scale_inherit_set(bool inherit_) const;

   /// @param camera The camera
   ///
   Evas_3D_Camera * camera_get() const;

   /// @param camera The camera
   ///
   void camera_set(Evas_3D_Camera * camera_) const;

   /// @param light The light
   ///
   Evas_3D_Light * light_get() const;

   /// @param light The light
   ///
   void light_set(Evas_3D_Light * light_) const;

   /// @param frame The animation frame number.
   ///
   int mesh_frame_get(Evas_3D_Mesh * mesh_) const;

   /// @param frame The animation frame number.
   ///
   void mesh_frame_set(Evas_3D_Mesh * mesh_, int frame_) const;

   /// @param target Target node.
   ///
   ::efl::eo::concrete billboard_target_get() const;

   /// @param target Target node.
   ///
   void billboard_target_set(::efl::eo::concrete target_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_NODE_CLASS);
   }

   operator ::evas_3d_node() const;
   operator ::evas_3d_node&();
   operator ::evas_3d_node const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas_3d_node*() const { return static_cast<::evas_3d_node*>(static_cast<D const*>(this)->p); }
      operator ::evas_3d_node const*() const { return static_cast<::evas_3d_node const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas_3d_node const*() const { return static_cast<::evas_3d_node const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class evas_3d_node
struct evas_3d_node
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas_3d_object)
{
   /// @cond LOCAL
   struct _c_constructor
   {

      explicit _c_constructor(Evas_3D_Node_Type type_)
         : type_(type_)
      {}
      void operator()()
      {
         ::evas_3d_node_constructor(type_);
      }
      void register_ev_del_free_callback(Eo* _eoptr)
      {
         (void) _eoptr;
      }

      Evas_3D_Node_Type type_;
   };
   /// @endcond

   //@{
   /**
      @brief Constructs a new evas_3d_node object.

      Constructs a new evas_3d_node object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Since this class have a necessary constructor method, you must call it
      in the right place within this constructor parameters.

      Example:
      @code
      evas_3d_node my_evas_3d_node(efl::eo::parent = parent_object,
         my_evas_3d_node.constructor(type_));
      @endcode

      @see evas_3d_node::constructor
      @see evas_3d_node(Eo* eo)
   */
   explicit evas_3d_node(::efl::eo::parent_type _p, _c_constructor _c0)
      : evas_3d_node(_ctors_call(_p, _c0))
   {}

   explicit evas_3d_node(_c_constructor _c0)
      : evas_3d_node(_ctors_call(::efl::eo::parent = nullptr, _c0))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit evas_3d_node(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit evas_3d_node(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   evas_3d_node(evas_3d_node const& other)
      : evas_3d_node(eo_ref(other._eo_ptr()))
   {}

   ~evas_3d_node() {}

   /// @brief Constructor.
   ///
   /// @param type 
   ///
   static _c_constructor constructor(Evas_3D_Node_Type type_);

   Evas_3D_Node_Type type_get() const;

   /// @param member Node object to be added.
   ///
   void member_add(Evas_3D_Node * member_) const;

   /// @param member Member node to be deleted from the given node.
   ///
   void member_del(Evas_3D_Node * member_) const;

   Evas_3D_Node * parent_get() const;

   ::efl::eina::crange_list< Evas_3D_Node * > member_list_get() const;

   /// @param x X coordinate of the position.
   /// @param y Y coordinate of the position.
   /// @param z Z coordinate of the position.
   ///
   void position_set(Evas_Real x_, Evas_Real y_, Evas_Real z_) const;

   /// @param x X term of the orientation quaternion (w, x, y, z)
   /// @param y Y term of the orientation quaternion (w, x, y, z)
   /// @param z Z term of the orientation quaternion (w, x, y, z)
   /// @param w W term of the orientation quaternion (w, x, y, z)
   ///
   void orientation_set(Evas_Real x_, Evas_Real y_, Evas_Real z_, Evas_Real w_) const;

   /// @param angle Rotation angle.
   /// @param x X term of the rotation axis.
   /// @param y Y term of the rotation axis.
   /// @param z Z term of the rotation axis.
   ///
   void orientation_angle_axis_set(Evas_Real angle_, Evas_Real x_, Evas_Real y_, Evas_Real z_) const;

   /// @param x Scale factor along X-axis.
   /// @param y Scale factor along Y-axis.
   /// @param z Scale factor along Z-axis.
   ///
   void scale_set(Evas_Real x_, Evas_Real y_, Evas_Real z_) const;

   /// @param space 
   /// @param[out] x Pointer to receive X coordinate of the position.
   /// @param[out] y Pointer to receive Y coordinate of the position.
   /// @param[out] z Pointer to receive Z coordinate of the position.
   ///
   void position_get(Evas_3D_Space space_, Evas_Real* x_, Evas_Real* y_, Evas_Real* z_) const;

   /// @param space 
   /// @param[out] x Pointer to receive X term of the orientation quaternion.
   /// @param[out] y Pointer to receive Y term of the orientation quaternion.
   /// @param[out] z Pointer to receive Z term of the orientation quaternion.
   /// @param[out] w Pointer to receive W term of the orientation quaternion.
   ///
   void orientation_get(Evas_3D_Space space_, Evas_Real* x_, Evas_Real* y_, Evas_Real* z_, Evas_Real* w_) const;

   /// @param space 
   /// @param[out] x Pointer to receive Scale factor along X-axis.
   /// @param[out] y Pointer to receive Scale factor along X-axis.
   /// @param[out] z Pointer to receive Scale factor along X-axis.
   ///
   void scale_get(Evas_3D_Space space_, Evas_Real* x_, Evas_Real* y_, Evas_Real* z_) const;

   /// @param target_space Space where the target position belongs to.
   /// @param x X coordinate of the target position.
   /// @param y Y coordinate of the target position.
   /// @param z Z coordinate of the target position.
   /// @param up_space Space where the up vector belongs to.
   /// @param ux X term of the up vector.
   /// @param uy Y term of the up vector.
   /// @param uz Z term of the up vector.
   ///
   void look_at_set(Evas_3D_Space target_space_, Evas_Real x_, Evas_Real y_, Evas_Real z_, Evas_3D_Space up_space_, Evas_Real ux_, Evas_Real uy_, Evas_Real uz_) const;

   /// @param mesh The mesh to be added.
   ///
   void mesh_add(Evas_3D_Mesh * mesh_) const;

   /// @param mesh The mesh to be deleted.
   ///
   void mesh_del(Evas_3D_Mesh * mesh_) const;

   ::efl::eina::crange_list< Evas_3D_Mesh * > mesh_list_get() const;

   /// @param x Coordinates of vector.
   /// @param y 
   /// @param z 
   /// @param x2 
   /// @param y2 
   /// @param z2 
   ///
   void bounding_box_get(Evas_Real * x_, Evas_Real * y_, Evas_Real * z_, Evas_Real * x2_, Evas_Real * y2_, Evas_Real * z2_) const;

   /// @param x Coordinates of vector.
   /// @param y 
   /// @param z 
   /// @param r 
   ///
   void bounding_sphere_get(Evas_Real * x_, Evas_Real * y_, Evas_Real * z_, Evas_Real * r_) const;

   /// @param inherit Whether to inherit parent position
   ///
   bool position_inherit_get() const;

   /// @param inherit Whether to inherit parent position
   ///
   void position_inherit_set(bool inherit_) const;

   /// @param inherit Whether to inherit parent orientation
   ///
   bool orientation_inherit_get() const;

   /// @param inherit Whether to inherit parent orientation
   ///
   void orientation_inherit_set(bool inherit_) const;

   /// @param inherit Whether to inherit parent scale
   ///
   bool scale_inherit_get() const;

   /// @param inherit Whether to inherit parent scale
   ///
   void scale_inherit_set(bool inherit_) const;

   /// @param camera The camera
   ///
   Evas_3D_Camera * camera_get() const;

   /// @param camera The camera
   ///
   void camera_set(Evas_3D_Camera * camera_) const;

   /// @param light The light
   ///
   Evas_3D_Light * light_get() const;

   /// @param light The light
   ///
   void light_set(Evas_3D_Light * light_) const;

   /// @param frame The animation frame number.
   ///
   int mesh_frame_get(Evas_3D_Mesh * mesh_) const;

   /// @param frame The animation frame number.
   ///
   void mesh_frame_set(Evas_3D_Mesh * mesh_, int frame_) const;

   /// @param target Target node.
   ///
   ::efl::eo::concrete billboard_target_get() const;

   /// @param target Target node.
   ///
   void billboard_target_set(::efl::eo::concrete target_) const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Clicked Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_clicked_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_CLICKED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>,
         EVAS_3D_OBJECT_EVENT_CLICKED );
   }

   /// Clicked Event
   template <typename T>
   void
   callback_clicked_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_CLICKED, info));
   }

   /// Collision Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_collision_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_COLLISION, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>,
         EVAS_3D_OBJECT_EVENT_COLLISION );
   }

   /// Collision Event
   template <typename T>
   void
   callback_collision_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_COLLISION, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_node, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_NODE_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas_3d_node::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas_3d_object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas_3d_node* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas_3d_node::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas_3d_object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas_3d_node const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p, _c_constructor _c0)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_3D_NODE_CLASS, _p._eo_raw, _c0());

      _c0.register_ev_del_free_callback(_ret_eo);
      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas_3d_node) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas_3d_node>::value, "");


#include "evas_3d_node.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_3D_NODE_HH

