#ifndef EFL_GENERATED_EVAS_3D_SCENE_HH
#define EFL_GENERATED_EVAS_3D_SCENE_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_3d_scene.eo.h"
}

#include "evas_3d_object.eo.hh"
#include "evas_common_interface.eo.hh"

struct evas_3d_scene;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct evas_3d_scene
{
   /// @param w Width of the resolution.
   /// @param h Height of the resolution.
   ///
   void size_set(int w_, int h_) const;

   /// @param[out] w Pointer to receive width of the resolution.
   /// @param[out] h Pointer to receive height of the resolution.
   ///
   void size_get(int* w_, int* h_) const;

   /// @param r Red component of the background color.
   /// @param g Green component of the background color.
   /// @param b Blue component of the background color.
   /// @param a Alpha component of the background color.
   ///
   void background_color_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @param[out] r Pointer to receive red component of the background color.
   /// @param[out] g Pointer to receive green component of the background color.
   /// @param[out] b Pointer to receive blue component of the background color.
   /// @param[out] a Pointer to receive alpha component of the background color.
   ///
   void background_color_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @param x X coordinate of the picking position.
   /// @param y Y coordinate of the picking position.
   /// @param[out] node Pointer to receive the node contains the picked mesh.
   /// @param[out] mesh Pointer to receive the picked mesh.
   /// @param[out] s Pointer to receive the texture "s" coordinate.
   /// @param[out] t Pointer to receive the texture "t" coordinate.
   ///
   bool pick(Evas_Real x_, Evas_Real y_, Evas_3D_Node ** node_, Evas_3D_Mesh ** mesh_, Evas_Real* s_, Evas_Real* t_) const;

   /// @param x X coordinate of the picking position.
   /// @param y Y coordinate of the picking position.
   /// @param node Node for search.
   ///
   Evas_3D_Node * exist(Evas_Real x_, Evas_Real y_, Evas_3D_Node * node_) const;

   /// @param x X coordinate of the picking position.
   /// @param y Y coordinate of the picking position.
   ///
   ::efl::eina::range_list< Evas_3D_Node * > pick_member_list_get(Evas_Real x_, Evas_Real y_) const;

   /// @param shadows_enabled shadows enabled status.
   ///
   void shadows_enable_set(bool shadows_enabled_) const;

   bool shadows_enable_get() const;

   bool color_pick_enable_get() const;

   /// @param color_pick Posibility flag
   ///
   bool color_pick_enable_set(bool color_pick_) const;

   /// @param node A node which will be used as a root node for the scene.
   ///
   Evas_3D_Node * root_node_get() const;

   /// @param node A node which will be used as a root node for the scene.
   ///
   void root_node_set(Evas_3D_Node * node_) const;

   /// @param node A node which will be used as a camera node for the scene.
   ///
   Evas_3D_Node * camera_node_get() const;

   /// @param node A node which will be used as a camera node for the scene.
   ///
   void camera_node_set(Evas_3D_Node * node_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_SCENE_CLASS);
   }

   operator ::evas_3d_scene() const;
   operator ::evas_3d_scene&();
   operator ::evas_3d_scene const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas_3d_scene*() const { return static_cast<::evas_3d_scene*>(static_cast<D const*>(this)->p); }
      operator ::evas_3d_scene const*() const { return static_cast<::evas_3d_scene const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas_3d_scene const*() const { return static_cast<::evas_3d_scene const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class evas_3d_scene
struct evas_3d_scene
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas_3d_object)
{
   //@{
   /**
      @brief Constructs a new evas_3d_scene object.

      Constructs a new evas_3d_scene object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas_3d_scene my_evas_3d_scene(efl::eo::parent = parent_object);
      @endcode

      @see evas_3d_scene(Eo* eo)
   */
   explicit evas_3d_scene(::efl::eo::parent_type _p)
      : evas_3d_scene(_ctors_call(_p))
   {}

   explicit evas_3d_scene()
      : evas_3d_scene(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit evas_3d_scene(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit evas_3d_scene(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   evas_3d_scene(evas_3d_scene const& other)
      : evas_3d_scene(eo_ref(other._eo_ptr()))
   {}

   ~evas_3d_scene() {}

   /// @param w Width of the resolution.
   /// @param h Height of the resolution.
   ///
   void size_set(int w_, int h_) const;

   /// @param[out] w Pointer to receive width of the resolution.
   /// @param[out] h Pointer to receive height of the resolution.
   ///
   void size_get(int* w_, int* h_) const;

   /// @param r Red component of the background color.
   /// @param g Green component of the background color.
   /// @param b Blue component of the background color.
   /// @param a Alpha component of the background color.
   ///
   void background_color_set(Evas_Real r_, Evas_Real g_, Evas_Real b_, Evas_Real a_) const;

   /// @param[out] r Pointer to receive red component of the background color.
   /// @param[out] g Pointer to receive green component of the background color.
   /// @param[out] b Pointer to receive blue component of the background color.
   /// @param[out] a Pointer to receive alpha component of the background color.
   ///
   void background_color_get(Evas_Real* r_, Evas_Real* g_, Evas_Real* b_, Evas_Real* a_) const;

   /// @param x X coordinate of the picking position.
   /// @param y Y coordinate of the picking position.
   /// @param[out] node Pointer to receive the node contains the picked mesh.
   /// @param[out] mesh Pointer to receive the picked mesh.
   /// @param[out] s Pointer to receive the texture "s" coordinate.
   /// @param[out] t Pointer to receive the texture "t" coordinate.
   ///
   bool pick(Evas_Real x_, Evas_Real y_, Evas_3D_Node ** node_, Evas_3D_Mesh ** mesh_, Evas_Real* s_, Evas_Real* t_) const;

   /// @param x X coordinate of the picking position.
   /// @param y Y coordinate of the picking position.
   /// @param node Node for search.
   ///
   Evas_3D_Node * exist(Evas_Real x_, Evas_Real y_, Evas_3D_Node * node_) const;

   /// @param x X coordinate of the picking position.
   /// @param y Y coordinate of the picking position.
   ///
   ::efl::eina::range_list< Evas_3D_Node * > pick_member_list_get(Evas_Real x_, Evas_Real y_) const;

   /// @param shadows_enabled shadows enabled status.
   ///
   void shadows_enable_set(bool shadows_enabled_) const;

   bool shadows_enable_get() const;

   bool color_pick_enable_get() const;

   /// @param color_pick Posibility flag
   ///
   bool color_pick_enable_set(bool color_pick_) const;

   /// @param node A node which will be used as a root node for the scene.
   ///
   Evas_3D_Node * root_node_get() const;

   /// @param node A node which will be used as a root node for the scene.
   ///
   void root_node_set(Evas_3D_Node * node_) const;

   /// @param node A node which will be used as a camera node for the scene.
   ///
   Evas_3D_Node * camera_node_get() const;

   /// @param node A node which will be used as a camera node for the scene.
   ///
   void camera_node_set(Evas_3D_Node * node_) const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Clicked Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_clicked_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_CLICKED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>,
         EVAS_3D_OBJECT_EVENT_CLICKED );
   }

   /// Clicked Event
   template <typename T>
   void
   callback_clicked_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_CLICKED, info));
   }

   /// Collision Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_collision_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_COLLISION, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>,
         EVAS_3D_OBJECT_EVENT_COLLISION );
   }

   /// Collision Event
   template <typename T>
   void
   callback_collision_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_COLLISION, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_scene, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_SCENE_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas_3d_scene::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas_3d_object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas_3d_scene* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas_3d_scene::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas_3d_object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas_3d_scene const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_3D_SCENE_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas_3d_scene) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas_3d_scene>::value, "");


#include "evas_3d_scene.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_3D_SCENE_HH

