#ifndef EFL_GENERATED_EVAS_3D_TEXTURE_HH
#define EFL_GENERATED_EVAS_3D_TEXTURE_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_3d_texture.eo.h"
}

#include "evas_3d_object.eo.hh"
#include "evas_common_interface.eo.hh"
#include <canvas/evas_object.eo.hh>
#include <string>

struct evas_3d_texture;

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

struct evas_3d_texture
{
   /// @param color_format Color format of the texture.
   /// @param pixel_format Pixel format of the data.
   /// @param w Width of the data.
   /// @param h Height of the data.
   /// @param data Pointer to the data.
   ///
   void data_set(Evas_3D_Color_Format color_format_, Evas_3D_Pixel_Format pixel_format_, int w_, int h_, const void * data_) const;

   /// @param file Path to the image file.
   /// @param key Key in the image file.
   ///
   void file_set(::efl::eina::string_view file_, ::efl::eina::string_view key_) const;

   /// @param source Source evas object to be used as the texture data.
   ///
   void source_set(::evas::object source_) const;

   Evas_3D_Color_Format color_format_get() const;

   /// @param[out] w Pointer to receive the width of the texture size.
   /// @param[out] h Pointer to receive the height of the texture size.
   ///
   void size_get(int* w_, int* h_) const;

   /// @param s Wrap mode for S-axis.
   /// @param t Wrap mode for T-axis.
   ///
   void wrap_set(Evas_3D_Wrap_Mode s_, Evas_3D_Wrap_Mode t_) const;

   /// @param[out] s Pointer to receive S-axis wrap mode.
   /// @param[out] t Pointer to receive T-axis wrap mode.
   ///
   void wrap_get(Evas_3D_Wrap_Mode* s_, Evas_3D_Wrap_Mode* t_) const;

   /// @param min Minification filter used when down-scaling.
   /// @param mag Magnification filter used when up-scaling.
   ///
   void filter_set(Evas_3D_Texture_Filter min_, Evas_3D_Texture_Filter mag_) const;

   /// @param[out] min Pointer to receive the minification filter.
   /// @param[out] mag Pointer to receive the magnification filter.
   ///
   void filter_get(Evas_3D_Texture_Filter* min_, Evas_3D_Texture_Filter* mag_) const;

   /// @param visible @c EINA_TRUE for visible, @c EINA_FALSE for invisible.
   ///
   bool source_visible_get() const;

   /// @param visible @c EINA_TRUE for visible, @c EINA_FALSE for invisible.
   ///
   void source_visible_set(bool visible_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_TEXTURE_CLASS);
   }

   operator ::evas_3d_texture() const;
   operator ::evas_3d_texture&();
   operator ::evas_3d_texture const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas_3d_texture*() const { return static_cast<::evas_3d_texture*>(static_cast<D const*>(this)->p); }
      operator ::evas_3d_texture const*() const { return static_cast<::evas_3d_texture const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas_3d_texture const*() const { return static_cast<::evas_3d_texture const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}
/// @endcond

/// @brief Class evas_3d_texture
struct evas_3d_texture
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas_3d_object)
{
   //@{
   /**
      @brief Constructs a new evas_3d_texture object.

      Constructs a new evas_3d_texture object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas_3d_texture my_evas_3d_texture(efl::eo::parent = parent_object);
      @endcode

      @see evas_3d_texture(Eo* eo)
   */
   explicit evas_3d_texture(::efl::eo::parent_type _p)
      : evas_3d_texture(_ctors_call(_p))
   {}

   explicit evas_3d_texture()
      : evas_3d_texture(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit evas_3d_texture(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit evas_3d_texture(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   evas_3d_texture(evas_3d_texture const& other)
      : evas_3d_texture(eo_ref(other._eo_ptr()))
   {}

   ~evas_3d_texture() {}

   /// @param color_format Color format of the texture.
   /// @param pixel_format Pixel format of the data.
   /// @param w Width of the data.
   /// @param h Height of the data.
   /// @param data Pointer to the data.
   ///
   void data_set(Evas_3D_Color_Format color_format_, Evas_3D_Pixel_Format pixel_format_, int w_, int h_, const void * data_) const;

   /// @param file Path to the image file.
   /// @param key Key in the image file.
   ///
   void file_set(::efl::eina::string_view file_, ::efl::eina::string_view key_) const;

   /// @param source Source evas object to be used as the texture data.
   ///
   void source_set(::evas::object source_) const;

   Evas_3D_Color_Format color_format_get() const;

   /// @param[out] w Pointer to receive the width of the texture size.
   /// @param[out] h Pointer to receive the height of the texture size.
   ///
   void size_get(int* w_, int* h_) const;

   /// @param s Wrap mode for S-axis.
   /// @param t Wrap mode for T-axis.
   ///
   void wrap_set(Evas_3D_Wrap_Mode s_, Evas_3D_Wrap_Mode t_) const;

   /// @param[out] s Pointer to receive S-axis wrap mode.
   /// @param[out] t Pointer to receive T-axis wrap mode.
   ///
   void wrap_get(Evas_3D_Wrap_Mode* s_, Evas_3D_Wrap_Mode* t_) const;

   /// @param min Minification filter used when down-scaling.
   /// @param mag Magnification filter used when up-scaling.
   ///
   void filter_set(Evas_3D_Texture_Filter min_, Evas_3D_Texture_Filter mag_) const;

   /// @param[out] min Pointer to receive the minification filter.
   /// @param[out] mag Pointer to receive the magnification filter.
   ///
   void filter_get(Evas_3D_Texture_Filter* min_, Evas_3D_Texture_Filter* mag_) const;

   /// @param visible @c EINA_TRUE for visible, @c EINA_FALSE for invisible.
   ///
   bool source_visible_get() const;

   /// @param visible @c EINA_TRUE for visible, @c EINA_FALSE for invisible.
   ///
   void source_visible_set(bool visible_) const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Clicked Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_clicked_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_CLICKED, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>,
         EVAS_3D_OBJECT_EVENT_CLICKED );
   }

   /// Clicked Event
   template <typename T>
   void
   callback_clicked_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_CLICKED, info));
   }

   /// Collision Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_collision_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_3D_OBJECT_EVENT_COLLISION, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>,
         EVAS_3D_OBJECT_EVENT_COLLISION );
   }

   /// Collision Event
   template <typename T>
   void
   callback_collision_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_3D_OBJECT_EVENT_COLLISION, info));
   }

   /// Obj is being deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas_3d_texture, function_type>,
         EO_BASE_EVENT_DEL );
   }

   /// Obj is being deleted.
   template <typename T>
   void
   callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_DEL, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_3D_TEXTURE_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas_3d_texture::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas_3d_object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas_3d_texture* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas_3d_texture::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas_3d_object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas_3d_texture const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_3D_TEXTURE_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas_3d_texture) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas_3d_texture>::value, "");


#include "evas_3d_texture.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_3D_TEXTURE_HH

