#ifndef EFL_GENERATED_EVAS_GRID_HH
#define EFL_GENERATED_EVAS_GRID_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_grid.eo.h"
}

#include "evas_smart_clipped.eo.hh"
#include <canvas/evas_object.eo.hh>

namespace evas {

struct grid;

}

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

namespace evas {

struct grid
{
   /// @brief Get an accessor to get random access to the list of children for the grid.
   ///
   /// @note Do not remove or delete objects while walking the list.
   /// @since 1.1
   ///
   Eina_Accessor * accessor_new() const;

   /// @brief Faster way to remove all child objects from a grid object.
   ///
   /// @since 1.1
   ///
   /// @param clear if true, it will delete just removed children.
   ///
   void clear(bool clear_) const;

   /// @brief Get an iterator to walk the list of children for the grid.
   ///
   /// @note Do not remove or delete objects while walking the list.
   /// @since 1.1
   ///
   Eina_Iterator * iterator_new() const;

   /// @brief Create a grid that is child of a given element @a parent.
   ///
   /// @see evas_object_grid_add()
   /// @since 1.1
   ///
   ::evas::object add_to() const;

   /// @brief Remove child from grid.
   ///
   /// @note removing a child will immediately call a walk over children in order
   /// to recalculate numbers of columns and rows. If you plan to remove
   /// all children, use evas_object_grid_clear() instead.
   ///
   /// @return 1 on success, 0 on failure.
   /// @since 1.1
   ///
   /// @param child 
   ///
   bool unpack(::evas::object child_) const;

   /// @brief Get the pack options for a grid child
   ///
   /// Get the pack x, y, width and height in virtual coordinates set by
   /// evas_object_grid_pack()
   /// @return 1 on success, 0 on failure.
   /// @since 1.1
   ///
   /// @param child The grid child to query for coordinates
   /// @param[out] x The pointer to where the x coordinate will be returned
   /// @param[out] y The pointer to where the y coordinate will be returned
   /// @param[out] w The pointer to where the width will be returned
   /// @param[out] h The pointer to where the height will be returned
   ///
   bool pack_get(::evas::object child_, int* x_, int* y_, int* w_, int* h_) const;

   /// @brief Add a new child to a grid object.
   ///
   /// @return 1 on success, 0 on failure.
   /// @since 1.1
   ///
   /// @param child The child object to add.
   /// @param x The virtual x coordinate of the child
   /// @param y The virtual y coordinate of the child
   /// @param w The virtual width of the child
   /// @param h The virtual height of the child
   ///
   bool pack(::evas::object child_, int x_, int y_, int w_, int h_) const;

   /// @brief Gets the mirrored mode of the grid.
   ///
   /// @return @c EINA_TRUE if it's a mirrored grid, @c EINA_FALSE otherwise.
   /// @see evas_object_grid_mirrored_set()
   /// @since 1.1
   ///
   /// @param mirrored the mirrored mode to set
   ///
   bool mirrored_get() const;

   /// @brief Sets the mirrored mode of the grid. In mirrored mode the grid items go
   /// from right to left instead of left to right. That is, 0,0 is top right, not
   /// to left.
   ///
   /// @since 1.1
   ///
   /// @param mirrored the mirrored mode to set
   ///
   void mirrored_set(bool mirrored_) const;

   /// @brief Get the current virtual resolution
   ///
   /// @see evas_object_grid_size_set()
   /// @since 1.1
   ///
   /// @param w The virtual horizontal size (resolution) in integer units
   /// @param h The virtual vertical size (resolution) in integer units
   ///
   void size_get(int* w_, int* h_) const;

   /// @brief Set the virtual resolution for the grid
   ///
   /// @since 1.1
   ///
   /// @param w The virtual horizontal size (resolution) in integer units
   /// @param h The virtual vertical size (resolution) in integer units
   ///
   void size_set(int w_, int h_) const;

   /// @brief Get the list of children for the grid.
   ///
   /// @note This is a duplicate of the list kept by the grid internally.
   /// It's up to the user to destroy it when it no longer needs it.
   /// It's possible to remove objects from the grid when walking this
   /// list, but these removals won't be reflected on it.
   /// @since 1.1
   ///
   ::efl::eina::list< ::evas::object > children_get() const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_GRID_CLASS);
   }

   operator ::evas::grid() const;
   operator ::evas::grid&();
   operator ::evas::grid const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas::grid*() const { return static_cast<::evas::grid*>(static_cast<D const*>(this)->p); }
      operator ::evas::grid const*() const { return static_cast<::evas::grid const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas::grid const*() const { return static_cast<::evas::grid const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}

}
/// @endcond

namespace evas {

/// @brief Class grid
struct grid
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(efl::gfx::base)
      , EO_CXX_INHERIT(efl::gfx::stack)
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas::object)
      , EO_CXX_INHERIT(evas::object_smart)
      , EO_CXX_INHERIT(evas::signal_interface)
      , EO_CXX_INHERIT(evas::smart_clipped)
{
   //@{
   /**
      @brief Constructs a new evas::grid object.

      Constructs a new evas::grid object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas::grid my_grid(efl::eo::parent = parent_object);
      @endcode

      @see grid(Eo* eo)
   */
   explicit grid(::efl::eo::parent_type _p)
      : grid(_ctors_call(_p))
   {}

   explicit grid()
      : grid(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit grid(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit grid(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   grid(grid const& other)
      : grid(eo_ref(other._eo_ptr()))
   {}

   ~grid() {}

   /// @brief Get an accessor to get random access to the list of children for the grid.
   ///
   /// @note Do not remove or delete objects while walking the list.
   /// @since 1.1
   ///
   Eina_Accessor * accessor_new() const;

   /// @brief Faster way to remove all child objects from a grid object.
   ///
   /// @since 1.1
   ///
   /// @param clear if true, it will delete just removed children.
   ///
   void clear(bool clear_) const;

   /// @brief Get an iterator to walk the list of children for the grid.
   ///
   /// @note Do not remove or delete objects while walking the list.
   /// @since 1.1
   ///
   Eina_Iterator * iterator_new() const;

   /// @brief Create a grid that is child of a given element @a parent.
   ///
   /// @see evas_object_grid_add()
   /// @since 1.1
   ///
   ::evas::object add_to() const;

   /// @brief Remove child from grid.
   ///
   /// @note removing a child will immediately call a walk over children in order
   /// to recalculate numbers of columns and rows. If you plan to remove
   /// all children, use evas_object_grid_clear() instead.
   ///
   /// @return 1 on success, 0 on failure.
   /// @since 1.1
   ///
   /// @param child 
   ///
   bool unpack(::evas::object child_) const;

   /// @brief Get the pack options for a grid child
   ///
   /// Get the pack x, y, width and height in virtual coordinates set by
   /// evas_object_grid_pack()
   /// @return 1 on success, 0 on failure.
   /// @since 1.1
   ///
   /// @param child The grid child to query for coordinates
   /// @param[out] x The pointer to where the x coordinate will be returned
   /// @param[out] y The pointer to where the y coordinate will be returned
   /// @param[out] w The pointer to where the width will be returned
   /// @param[out] h The pointer to where the height will be returned
   ///
   bool pack_get(::evas::object child_, int* x_, int* y_, int* w_, int* h_) const;

   /// @brief Add a new child to a grid object.
   ///
   /// @return 1 on success, 0 on failure.
   /// @since 1.1
   ///
   /// @param child The child object to add.
   /// @param x The virtual x coordinate of the child
   /// @param y The virtual y coordinate of the child
   /// @param w The virtual width of the child
   /// @param h The virtual height of the child
   ///
   bool pack(::evas::object child_, int x_, int y_, int w_, int h_) const;

   /// @brief Gets the mirrored mode of the grid.
   ///
   /// @return @c EINA_TRUE if it's a mirrored grid, @c EINA_FALSE otherwise.
   /// @see evas_object_grid_mirrored_set()
   /// @since 1.1
   ///
   /// @param mirrored the mirrored mode to set
   ///
   bool mirrored_get() const;

   /// @brief Sets the mirrored mode of the grid. In mirrored mode the grid items go
   /// from right to left instead of left to right. That is, 0,0 is top right, not
   /// to left.
   ///
   /// @since 1.1
   ///
   /// @param mirrored the mirrored mode to set
   ///
   void mirrored_set(bool mirrored_) const;

   /// @brief Get the current virtual resolution
   ///
   /// @see evas_object_grid_size_set()
   /// @since 1.1
   ///
   /// @param w The virtual horizontal size (resolution) in integer units
   /// @param h The virtual vertical size (resolution) in integer units
   ///
   void size_get(int* w_, int* h_) const;

   /// @brief Set the virtual resolution for the grid
   ///
   /// @since 1.1
   ///
   /// @param w The virtual horizontal size (resolution) in integer units
   /// @param h The virtual vertical size (resolution) in integer units
   ///
   void size_set(int w_, int h_) const;

   /// @brief Get the list of children for the grid.
   ///
   /// @note This is a duplicate of the list kept by the grid internally.
   /// It's up to the user to destroy it when it no longer needs it.
   /// It's possible to remove objects from the grid when walking this
   /// list, but these removals won't be reflected on it.
   /// @since 1.1
   ///
   ::efl::eina::list< ::evas::object > children_get() const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Size hints changed event
   template <typename F>
   ::efl::eo::signal_connection
   callback_changed_size_hints_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS );
   }

   /// Size hints changed event
   template <typename T>
   void
   callback_changed_size_hints_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS, info));
   }

   /// Focus In Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_focus_in_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FOCUS_IN, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_FOCUS_IN );
   }

   /// Focus In Event
   template <typename T>
   void
   callback_focus_in_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FOCUS_IN, info));
   }

   /// Focus Out Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_focus_out_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FOCUS_OUT, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_FOCUS_OUT );
   }

   /// Focus Out Event
   template <typename T>
   void
   callback_focus_out_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FOCUS_OUT, info));
   }

   /// Object Being Freed (Called after Del)
   template <typename F>
   ::efl::eo::signal_connection
   callback_free_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FREE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_FREE );
   }

   /// Object Being Freed (Called after Del)
   template <typename T>
   void
   callback_free_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FREE, info));
   }

   /// Hide Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_hide_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_HIDE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_HIDE );
   }

   /// Hide Event
   template <typename T>
   void
   callback_hide_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_HIDE, info));
   }

   /// Events go on/off hold
   template <typename F>
   ::efl::eo::signal_connection
   callback_hold_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_HOLD, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_HOLD );
   }

   /// Events go on/off hold
   template <typename T>
   void
   callback_hold_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_HOLD, info));
   }

   /// Image has been preloaded
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_preloaded_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_PRELOADED, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_PRELOADED );
   }

   /// Image has been preloaded
   template <typename T>
   void
   callback_image_preloaded_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_PRELOADED, info));
   }

   /// Image resize
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_resize_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_RESIZE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_RESIZE );
   }

   /// Image resize
   template <typename T>
   void
   callback_image_resize_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_RESIZE, info));
   }

   /// Image data has been unloaded (by some mechanism in Evas that throw out original image data)
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_unloaded_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_UNLOADED, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_UNLOADED );
   }

   /// Image data has been unloaded (by some mechanism in Evas that throw out original image data)
   template <typename T>
   void
   callback_image_unloaded_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_UNLOADED, info));
   }

   /// Key Press Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_key_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_KEY_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_KEY_DOWN );
   }

   /// Key Press Event
   template <typename T>
   void
   callback_key_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_KEY_DOWN, info));
   }

   /// Key Release Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_key_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_KEY_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_KEY_UP );
   }

   /// Key Release Event
   template <typename T>
   void
   callback_key_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_KEY_UP, info));
   }

   /// Mouse Button Down Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_DOWN );
   }

   /// Mouse Button Down Event
   template <typename T>
   void
   callback_mouse_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_DOWN, info));
   }

   /// Mouse In Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_in_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_IN, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_IN );
   }

   /// Mouse In Event
   template <typename T>
   void
   callback_mouse_in_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_IN, info));
   }

   /// Mouse Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_MOVE );
   }

   /// Mouse Move Event
   template <typename T>
   void
   callback_mouse_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_MOVE, info));
   }

   /// Mouse Out Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_out_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_OUT, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_OUT );
   }

   /// Mouse Out Event
   template <typename T>
   void
   callback_mouse_out_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_OUT, info));
   }

   /// Mouse Button Up Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_UP );
   }

   /// Mouse Button Up Event
   template <typename T>
   void
   callback_mouse_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_UP, info));
   }

   /// Mouse Wheel Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_wheel_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_WHEEL, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_WHEEL );
   }

   /// Mouse Wheel Event
   template <typename T>
   void
   callback_mouse_wheel_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_WHEEL, info));
   }

   /// Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MOVE );
   }

   /// Move Event
   template <typename T>
   void
   callback_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOVE, info));
   }

   /// Mouse-touch Down Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MULTI_DOWN );
   }

   /// Mouse-touch Down Event
   template <typename T>
   void
   callback_multi_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_DOWN, info));
   }

   /// Multi-touch Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MULTI_MOVE );
   }

   /// Multi-touch Move Event
   template <typename T>
   void
   callback_multi_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_MOVE, info));
   }

   /// Mouse-touch Up Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_MULTI_UP );
   }

   /// Mouse-touch Up Event
   template <typename T>
   void
   callback_multi_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_UP, info));
   }

   /// Resize Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_resize_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_RESIZE, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_RESIZE );
   }

   /// Resize Event
   template <typename T>
   void
   callback_resize_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_RESIZE, info));
   }

   /// Restack Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_restack_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_RESTACK, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_RESTACK );
   }

   /// Restack Event
   template <typename T>
   void
   callback_restack_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_RESTACK, info));
   }

   /// Show Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_show_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_SHOW, priority_,
            &::efl::eo::_detail::event_callback<::evas::grid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::grid, function_type>,
         EVAS_OBJECT_EVENT_SHOW );
   }

   /// Show Event
   template <typename T>
   void
   callback_show_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_SHOW, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_GRID_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas::grid::address_of<address_of>
      , ::eo_cxx::efl::gfx::base::address_of<address_of>
      , ::eo_cxx::efl::gfx::stack::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas::object::address_of<address_of>
      , ::eo_cxx::evas::object_smart::address_of<address_of>
      , ::eo_cxx::evas::signal_interface::address_of<address_of>
      , ::eo_cxx::evas::smart_clipped::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas::grid* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas::grid::address_const_of<address_const_of>
      , ::eo_cxx::efl::gfx::base::address_const_of<address_const_of>
      , ::eo_cxx::efl::gfx::stack::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas::object::address_const_of<address_const_of>
      , ::eo_cxx::evas::object_smart::address_const_of<address_const_of>
      , ::eo_cxx::evas::signal_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas::smart_clipped::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas::grid const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_GRID_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas::grid) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas::grid>::value, "");

}


#include "evas_grid.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_GRID_HH

