#ifndef EFL_GENERATED_EVAS_OBJECT_SMART_HH
#define EFL_GENERATED_EVAS_OBJECT_SMART_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_object_smart.eo.h"
}

#include "evas_object.eo.hh"
#include "evas_signal_interface.eo.hh"
#include <canvas/evas_object.eo.hh>
#include <string>

namespace evas {

struct object_smart;

}

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

namespace evas {

struct object_smart
{
   /// @brief No description supplied by the EAPI.
   ///
   void show() const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param x 
   /// @param y 
   ///
   void move(Evas_Coord x_, Evas_Coord y_) const;

   /// @brief Set an smart object @b instance's smart callbacks descriptions.
   ///
   /// @return @c EINA_TRUE on success, @c EINA_FALSE on failure.
   ///
   /// These descriptions are hints to be used by introspection and are
   /// not enforced in any way.
   ///
   /// It will not be checked if instance callbacks descriptions have the
   /// same name as respective possibly registered in the smart object
   /// @b class. Both are kept in different arrays and users of
   /// evas_object_smart_callbacks_descriptions_get() should handle this
   /// case as they wish.
   ///
   /// @note Becase @p descriptions must be @c NULL terminated, and
   /// because a @c NULL name makes little sense, too,
   /// Evas_Smart_Cb_Description.name must @b not be @c NULL.
   ///
   /// @note While instance callbacks descriptions are possible, they are
   /// @b not recommended. Use @b class callbacks descriptions
   /// instead as they make you smart object user's life simpler and
   /// will use less memory, as descriptions and arrays will be
   /// shared among all instances.
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param descriptions @c NULL terminated array with
   /// #Evas_Smart_Cb_Description descriptions. Array elements won't be
   /// modified at run time, but references to them and their contents
   /// will be made, so this array should be kept alive during the whole
   /// object's lifetime.
   ///
   bool callbacks_descriptions_set(const Evas_Smart_Cb_Description * descriptions_) const;

   /// @brief Retrieve an smart object's know smart callback descriptions (both
   /// instance and class ones).
   ///
   /// This call searches for registered callback descriptions for both
   /// instance and class of the given smart object. These arrays will be
   /// sorted by Evas_Smart_Cb_Description.name and also @c NULL
   /// terminated, so both @a class_count and @a instance_count can be
   /// ignored, if the caller wishes so. The terminator @c NULL is not
   /// counted in these values.
   ///
   /// @note If just class descriptions are of interest, try
   /// evas_smart_callbacks_descriptions_get() instead.
   ///
   /// @note Use @c NULL pointers on the descriptions/counters you're not
   /// interested in: they'll be ignored by the function.
   ///
   /// @see evas_smart_callbacks_descriptions_get()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param[out] class_descriptions Where to store class callbacks
   /// descriptions array, if any is known. If no descriptions are
   /// known, @c NULL is returned
   /// @param[out] class_count Returns how many class callbacks descriptions
   /// are known.
   /// @param[out] instance_descriptions Where to store instance callbacks
   /// descriptions array, if any is known. If no descriptions are
   /// known, @c NULL is returned.
   /// @param[out] instance_count Returns how many instance callbacks
   /// descriptions are known.
   ///
   void callbacks_descriptions_get(const Evas_Smart_Cb_Description *** class_descriptions_, unsigned int* class_count_, const Evas_Smart_Cb_Description *** instance_descriptions_, unsigned int* instance_count_) const;

   /// @brief Retrieves an iterator of the member objects of a given Evas smart
   /// object
   ///
   /// @return Returns the iterator of the member objects of @p obj.
   ///
   /// @since 1.8
   ///
   /// @see evas_object_smart_member_add()
   /// @see evas_object_smart_member_del()
   /// @see evas_object_smart_members_get()
   ///
   Eina_Iterator * iterator_new() const;

   /// @brief Find callback description for callback called @a name.
   ///
   /// or @c NULL if not found. If parameter is @c NULL, no search
   /// will be done on instance descriptions.
   /// @return reference to description if found, @c NULL if not found.
   ///
   /// @param name name of desired callback, must @b not be @c NULL.  The
   /// search have a special case for @a name being the same
   /// pointer as registered with Evas_Smart_Cb_Description, one
   /// can use it to avoid excessive use of strcmp().
   /// @param[out] class_description pointer to return class description or
   /// @c NULL if not found. If parameter is @c NULL, no search will
   /// be done on class descriptions.
   /// @param[out] instance_description pointer to return instance description
   ///
   void callback_description_find(::efl::eina::string_view name_, const Evas_Smart_Cb_Description ** class_description_, const Evas_Smart_Cb_Description ** instance_description_) const;

   /// @brief No description supplied by the EAPI.
   ///
   void hide() const;

   /// @brief Call the @b calculate() smart function immediately on a given smart
   /// object.
   ///
   /// This will force immediate calculations (see #Evas_Smart_Class)
   /// needed for renderization of this object and, besides, unset the
   /// flag on it telling it needs recalculation for the next rendering
   /// phase.
   ///
   /// @see evas_object_smart_need_recalculate_set()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   void calculate() const;

   /// @brief Instantiates a new smart object described by @p s.
   ///
   /// @return a new #Evas_Object handle
   ///
   /// This is the function one should use when defining the public
   /// function @b adding an instance of the new smart object to a given
   /// canvas. It will take care of setting all of its internals to work
   /// as they should, if the user set things properly, as seem on the
   /// #EVAS_SMART_SUBCLASS_NEW, for example.
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   void add() const;

   /// @brief Set an Evas object as a member of a given smart object.
   ///
   /// Members will automatically be stacked and layered together with the
   /// smart object. The various stacking functions will operate on
   /// members relative to the other members instead of the entire canvas,
   /// since they now live on an exclusive layer (see
   /// evas_object_stack_above(), for more details).
   ///
   /// Any @p smart_obj object's specific implementation of the @c
   /// member_add() smart function will take place too, naturally.
   ///
   /// @see evas_object_smart_member_del()
   /// @see evas_object_smart_members_get()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param sub_obj The member object
   ///
   void member_add(::evas::object sub_obj_) const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param w 
   /// @param h 
   ///
   void resize(Evas_Coord w_, Evas_Coord h_) const;

   /// @brief No description supplied by the EAPI.
   ///
   void clip_unset() const;

   /// @brief Mark smart object as changed, dirty.
   ///
   /// This will flag the given object as needing recalculation,
   /// forcefully. As an effect, on the next rendering cycle its @b
   /// calculate() (see #Evas_Smart_Class) smart function will be called.
   ///
   /// @see evas_object_smart_need_recalculate_set().
   /// @see evas_object_smart_calculate().
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   void changed() const;

   /// @brief Removes a member object from a given smart object.
   ///
   /// This removes a member object from a smart object, if it was added
   /// to any. The object will still be on the canvas, but no longer
   /// associated with whichever smart object it was associated with.
   ///
   /// @see evas_object_smart_member_add() for more details
   /// @see evas_object_smart_members_get()
   ///
   /// @param sub_obj the member object
   /// @ingroup Evas_Smart_Object_Group
   ///
   void member_del(::evas::object sub_obj_) const;

   /// @brief Deletes a smart object.
   ///
   void del() const;

   /// @brief Attach a given smart data to a given smart object.
   ///
   /// @param s 
   ///
   void attach(Evas_Smart * s_) const;

   /// @brief Get the value of the flag signalling that a given smart object needs to
   /// get recalculated.
   ///
   /// @return if flag is set or not.
   ///
   /// @note this flag will be unset during the rendering phase, when the
   /// @c calculate() smart function is called, if one is provided.
   /// If it's not provided, then the flag will be left unchanged
   /// after the rendering phase.
   ///
   /// @see evas_object_smart_need_recalculate_set(), for more details
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param value whether one wants to set (@c EINA_TRUE) or to unset
   /// (@c EINA_FALSE) the flag.
   ///
   bool need_recalculate_get() const;

   /// @brief Set or unset the flag signalling that a given smart object needs to
   /// get recalculated.
   ///
   /// If this flag is set, then the @c calculate() smart function of @p
   /// obj will be called, if one is provided, during rendering phase of
   /// Evas (see evas_render()), after which this flag will be
   /// automatically unset.
   ///
   /// If that smart function is not provided for the given object, this
   /// flag will be left unchanged.
   ///
   /// @note just setting this flag will not make the canvas' whole scene
   /// dirty, by itself, and evas_render() will have no effect. To
   /// force that, use evas_object_smart_changed(), that will also
   /// call this function automatically, with @c EINA_TRUE as parameter.
   ///
   /// @see evas_object_smart_need_recalculate_get()
   /// @see evas_object_smart_calculate()
   /// @see evas_smart_objects_calculate()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param value whether one wants to set (@c EINA_TRUE) or to unset
   /// (@c EINA_FALSE) the flag.
   ///
   void need_recalculate_set(bool value_) const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param clip 
   ///
   void clip_set(::evas::object clip_) const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param r 
   /// @param g 
   /// @param b 
   /// @param a 
   ///
   void color_set(int r_, int g_, int b_, int a_) const;

   /// @brief Store a pointer to user data for a given smart object.
   ///
   /// This data is stored @b independently of the one set by
   /// evas_object_data_set(), naturally.
   ///
   /// @see evas_object_smart_data_get()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param data A pointer to user data
   ///
   void data_set(void * data_) const;

   /// @brief Retrieves the list of the member objects of a given Evas smart
   /// object
   ///
   /// @return Returns the list of the member objects of @p obj.
   ///
   /// The returned list should be freed with @c eina_list_free() when you
   /// no longer need it.
   ///
   /// @since 1.7 This function will return @c NULL when a non-smart object is passed.
   ///
   /// @see evas_object_smart_member_add()
   /// @see evas_object_smart_member_del()
   /// @see evas_object_smart_iterator_new()
   ///
   ::efl::eina::range_list< ::evas::object > members_get() const;

   /// @brief Get the #Evas_Smart from which @p obj smart object was created.
   ///
   /// @return the #Evas_Smart handle or @c NULL, on errors
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   Evas_Smart * smart_get() const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_OBJECT_SMART_CLASS);
   }

   operator ::evas::object_smart() const;
   operator ::evas::object_smart&();
   operator ::evas::object_smart const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas::object_smart*() const { return static_cast<::evas::object_smart*>(static_cast<D const*>(this)->p); }
      operator ::evas::object_smart const*() const { return static_cast<::evas::object_smart const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas::object_smart const*() const { return static_cast<::evas::object_smart const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}

}
/// @endcond

namespace evas {

/// @brief Class object_smart
struct object_smart
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(efl::gfx::base)
      , EO_CXX_INHERIT(efl::gfx::stack)
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas::object)
      , EO_CXX_INHERIT(evas::signal_interface)
{
   //@{
   /**
      @brief Constructs a new evas::object_smart object.

      Constructs a new evas::object_smart object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas::object_smart my_object_smart(efl::eo::parent = parent_object);
      @endcode

      @see object_smart(Eo* eo)
   */
   explicit object_smart(::efl::eo::parent_type _p)
      : object_smart(_ctors_call(_p))
   {}

   explicit object_smart()
      : object_smart(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit object_smart(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit object_smart(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   object_smart(object_smart const& other)
      : object_smart(eo_ref(other._eo_ptr()))
   {}

   ~object_smart() {}

   /// @brief No description supplied by the EAPI.
   ///
   void show() const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param x 
   /// @param y 
   ///
   void move(Evas_Coord x_, Evas_Coord y_) const;

   /// @brief Set an smart object @b instance's smart callbacks descriptions.
   ///
   /// @return @c EINA_TRUE on success, @c EINA_FALSE on failure.
   ///
   /// These descriptions are hints to be used by introspection and are
   /// not enforced in any way.
   ///
   /// It will not be checked if instance callbacks descriptions have the
   /// same name as respective possibly registered in the smart object
   /// @b class. Both are kept in different arrays and users of
   /// evas_object_smart_callbacks_descriptions_get() should handle this
   /// case as they wish.
   ///
   /// @note Becase @p descriptions must be @c NULL terminated, and
   /// because a @c NULL name makes little sense, too,
   /// Evas_Smart_Cb_Description.name must @b not be @c NULL.
   ///
   /// @note While instance callbacks descriptions are possible, they are
   /// @b not recommended. Use @b class callbacks descriptions
   /// instead as they make you smart object user's life simpler and
   /// will use less memory, as descriptions and arrays will be
   /// shared among all instances.
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param descriptions @c NULL terminated array with
   /// #Evas_Smart_Cb_Description descriptions. Array elements won't be
   /// modified at run time, but references to them and their contents
   /// will be made, so this array should be kept alive during the whole
   /// object's lifetime.
   ///
   bool callbacks_descriptions_set(const Evas_Smart_Cb_Description * descriptions_) const;

   /// @brief Retrieve an smart object's know smart callback descriptions (both
   /// instance and class ones).
   ///
   /// This call searches for registered callback descriptions for both
   /// instance and class of the given smart object. These arrays will be
   /// sorted by Evas_Smart_Cb_Description.name and also @c NULL
   /// terminated, so both @a class_count and @a instance_count can be
   /// ignored, if the caller wishes so. The terminator @c NULL is not
   /// counted in these values.
   ///
   /// @note If just class descriptions are of interest, try
   /// evas_smart_callbacks_descriptions_get() instead.
   ///
   /// @note Use @c NULL pointers on the descriptions/counters you're not
   /// interested in: they'll be ignored by the function.
   ///
   /// @see evas_smart_callbacks_descriptions_get()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param[out] class_descriptions Where to store class callbacks
   /// descriptions array, if any is known. If no descriptions are
   /// known, @c NULL is returned
   /// @param[out] class_count Returns how many class callbacks descriptions
   /// are known.
   /// @param[out] instance_descriptions Where to store instance callbacks
   /// descriptions array, if any is known. If no descriptions are
   /// known, @c NULL is returned.
   /// @param[out] instance_count Returns how many instance callbacks
   /// descriptions are known.
   ///
   void callbacks_descriptions_get(const Evas_Smart_Cb_Description *** class_descriptions_, unsigned int* class_count_, const Evas_Smart_Cb_Description *** instance_descriptions_, unsigned int* instance_count_) const;

   /// @brief Retrieves an iterator of the member objects of a given Evas smart
   /// object
   ///
   /// @return Returns the iterator of the member objects of @p obj.
   ///
   /// @since 1.8
   ///
   /// @see evas_object_smart_member_add()
   /// @see evas_object_smart_member_del()
   /// @see evas_object_smart_members_get()
   ///
   Eina_Iterator * iterator_new() const;

   /// @brief Find callback description for callback called @a name.
   ///
   /// or @c NULL if not found. If parameter is @c NULL, no search
   /// will be done on instance descriptions.
   /// @return reference to description if found, @c NULL if not found.
   ///
   /// @param name name of desired callback, must @b not be @c NULL.  The
   /// search have a special case for @a name being the same
   /// pointer as registered with Evas_Smart_Cb_Description, one
   /// can use it to avoid excessive use of strcmp().
   /// @param[out] class_description pointer to return class description or
   /// @c NULL if not found. If parameter is @c NULL, no search will
   /// be done on class descriptions.
   /// @param[out] instance_description pointer to return instance description
   ///
   void callback_description_find(::efl::eina::string_view name_, const Evas_Smart_Cb_Description ** class_description_, const Evas_Smart_Cb_Description ** instance_description_) const;

   /// @brief No description supplied by the EAPI.
   ///
   void hide() const;

   /// @brief Call the @b calculate() smart function immediately on a given smart
   /// object.
   ///
   /// This will force immediate calculations (see #Evas_Smart_Class)
   /// needed for renderization of this object and, besides, unset the
   /// flag on it telling it needs recalculation for the next rendering
   /// phase.
   ///
   /// @see evas_object_smart_need_recalculate_set()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   void calculate() const;

   /// @brief Instantiates a new smart object described by @p s.
   ///
   /// @return a new #Evas_Object handle
   ///
   /// This is the function one should use when defining the public
   /// function @b adding an instance of the new smart object to a given
   /// canvas. It will take care of setting all of its internals to work
   /// as they should, if the user set things properly, as seem on the
   /// #EVAS_SMART_SUBCLASS_NEW, for example.
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   void add() const;

   /// @brief Set an Evas object as a member of a given smart object.
   ///
   /// Members will automatically be stacked and layered together with the
   /// smart object. The various stacking functions will operate on
   /// members relative to the other members instead of the entire canvas,
   /// since they now live on an exclusive layer (see
   /// evas_object_stack_above(), for more details).
   ///
   /// Any @p smart_obj object's specific implementation of the @c
   /// member_add() smart function will take place too, naturally.
   ///
   /// @see evas_object_smart_member_del()
   /// @see evas_object_smart_members_get()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param sub_obj The member object
   ///
   void member_add(::evas::object sub_obj_) const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param w 
   /// @param h 
   ///
   void resize(Evas_Coord w_, Evas_Coord h_) const;

   /// @brief No description supplied by the EAPI.
   ///
   void clip_unset() const;

   /// @brief Mark smart object as changed, dirty.
   ///
   /// This will flag the given object as needing recalculation,
   /// forcefully. As an effect, on the next rendering cycle its @b
   /// calculate() (see #Evas_Smart_Class) smart function will be called.
   ///
   /// @see evas_object_smart_need_recalculate_set().
   /// @see evas_object_smart_calculate().
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   void changed() const;

   /// @brief Removes a member object from a given smart object.
   ///
   /// This removes a member object from a smart object, if it was added
   /// to any. The object will still be on the canvas, but no longer
   /// associated with whichever smart object it was associated with.
   ///
   /// @see evas_object_smart_member_add() for more details
   /// @see evas_object_smart_members_get()
   ///
   /// @param sub_obj the member object
   /// @ingroup Evas_Smart_Object_Group
   ///
   void member_del(::evas::object sub_obj_) const;

   /// @brief Deletes a smart object.
   ///
   void del() const;

   /// @brief Attach a given smart data to a given smart object.
   ///
   /// @param s 
   ///
   void attach(Evas_Smart * s_) const;

   /// @brief Get the value of the flag signalling that a given smart object needs to
   /// get recalculated.
   ///
   /// @return if flag is set or not.
   ///
   /// @note this flag will be unset during the rendering phase, when the
   /// @c calculate() smart function is called, if one is provided.
   /// If it's not provided, then the flag will be left unchanged
   /// after the rendering phase.
   ///
   /// @see evas_object_smart_need_recalculate_set(), for more details
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param value whether one wants to set (@c EINA_TRUE) or to unset
   /// (@c EINA_FALSE) the flag.
   ///
   bool need_recalculate_get() const;

   /// @brief Set or unset the flag signalling that a given smart object needs to
   /// get recalculated.
   ///
   /// If this flag is set, then the @c calculate() smart function of @p
   /// obj will be called, if one is provided, during rendering phase of
   /// Evas (see evas_render()), after which this flag will be
   /// automatically unset.
   ///
   /// If that smart function is not provided for the given object, this
   /// flag will be left unchanged.
   ///
   /// @note just setting this flag will not make the canvas' whole scene
   /// dirty, by itself, and evas_render() will have no effect. To
   /// force that, use evas_object_smart_changed(), that will also
   /// call this function automatically, with @c EINA_TRUE as parameter.
   ///
   /// @see evas_object_smart_need_recalculate_get()
   /// @see evas_object_smart_calculate()
   /// @see evas_smart_objects_calculate()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param value whether one wants to set (@c EINA_TRUE) or to unset
   /// (@c EINA_FALSE) the flag.
   ///
   void need_recalculate_set(bool value_) const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param clip 
   ///
   void clip_set(::evas::object clip_) const;

   /// @brief No description supplied by the EAPI.
   ///
   /// @param r 
   /// @param g 
   /// @param b 
   /// @param a 
   ///
   void color_set(int r_, int g_, int b_, int a_) const;

   /// @brief Store a pointer to user data for a given smart object.
   ///
   /// This data is stored @b independently of the one set by
   /// evas_object_data_set(), naturally.
   ///
   /// @see evas_object_smart_data_get()
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   /// @param data A pointer to user data
   ///
   void data_set(void * data_) const;

   /// @brief Retrieves the list of the member objects of a given Evas smart
   /// object
   ///
   /// @return Returns the list of the member objects of @p obj.
   ///
   /// The returned list should be freed with @c eina_list_free() when you
   /// no longer need it.
   ///
   /// @since 1.7 This function will return @c NULL when a non-smart object is passed.
   ///
   /// @see evas_object_smart_member_add()
   /// @see evas_object_smart_member_del()
   /// @see evas_object_smart_iterator_new()
   ///
   ::efl::eina::range_list< ::evas::object > members_get() const;

   /// @brief Get the #Evas_Smart from which @p obj smart object was created.
   ///
   /// @return the #Evas_Smart handle or @c NULL, on errors
   ///
   /// @ingroup Evas_Smart_Object_Group
   ///
   Evas_Smart * smart_get() const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Size hints changed event
   template <typename F>
   ::efl::eo::signal_connection
   callback_changed_size_hints_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS );
   }

   /// Size hints changed event
   template <typename T>
   void
   callback_changed_size_hints_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS, info));
   }

   /// Focus In Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_focus_in_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FOCUS_IN, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_FOCUS_IN );
   }

   /// Focus In Event
   template <typename T>
   void
   callback_focus_in_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FOCUS_IN, info));
   }

   /// Focus Out Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_focus_out_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FOCUS_OUT, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_FOCUS_OUT );
   }

   /// Focus Out Event
   template <typename T>
   void
   callback_focus_out_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FOCUS_OUT, info));
   }

   /// Object Being Freed (Called after Del)
   template <typename F>
   ::efl::eo::signal_connection
   callback_free_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FREE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_FREE );
   }

   /// Object Being Freed (Called after Del)
   template <typename T>
   void
   callback_free_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FREE, info));
   }

   /// Hide Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_hide_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_HIDE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_HIDE );
   }

   /// Hide Event
   template <typename T>
   void
   callback_hide_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_HIDE, info));
   }

   /// Events go on/off hold
   template <typename F>
   ::efl::eo::signal_connection
   callback_hold_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_HOLD, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_HOLD );
   }

   /// Events go on/off hold
   template <typename T>
   void
   callback_hold_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_HOLD, info));
   }

   /// Image has been preloaded
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_preloaded_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_PRELOADED, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_PRELOADED );
   }

   /// Image has been preloaded
   template <typename T>
   void
   callback_image_preloaded_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_PRELOADED, info));
   }

   /// Image resize
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_resize_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_RESIZE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_RESIZE );
   }

   /// Image resize
   template <typename T>
   void
   callback_image_resize_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_RESIZE, info));
   }

   /// Image data has been unloaded (by some mechanism in Evas that throw out original image data)
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_unloaded_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_UNLOADED, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_UNLOADED );
   }

   /// Image data has been unloaded (by some mechanism in Evas that throw out original image data)
   template <typename T>
   void
   callback_image_unloaded_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_UNLOADED, info));
   }

   /// Key Press Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_key_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_KEY_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_KEY_DOWN );
   }

   /// Key Press Event
   template <typename T>
   void
   callback_key_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_KEY_DOWN, info));
   }

   /// Key Release Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_key_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_KEY_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_KEY_UP );
   }

   /// Key Release Event
   template <typename T>
   void
   callback_key_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_KEY_UP, info));
   }

   /// Mouse Button Down Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_DOWN );
   }

   /// Mouse Button Down Event
   template <typename T>
   void
   callback_mouse_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_DOWN, info));
   }

   /// Mouse In Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_in_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_IN, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_IN );
   }

   /// Mouse In Event
   template <typename T>
   void
   callback_mouse_in_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_IN, info));
   }

   /// Mouse Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_MOVE );
   }

   /// Mouse Move Event
   template <typename T>
   void
   callback_mouse_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_MOVE, info));
   }

   /// Mouse Out Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_out_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_OUT, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_OUT );
   }

   /// Mouse Out Event
   template <typename T>
   void
   callback_mouse_out_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_OUT, info));
   }

   /// Mouse Button Up Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_UP );
   }

   /// Mouse Button Up Event
   template <typename T>
   void
   callback_mouse_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_UP, info));
   }

   /// Mouse Wheel Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_wheel_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_WHEEL, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_WHEEL );
   }

   /// Mouse Wheel Event
   template <typename T>
   void
   callback_mouse_wheel_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_WHEEL, info));
   }

   /// Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MOVE );
   }

   /// Move Event
   template <typename T>
   void
   callback_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOVE, info));
   }

   /// Mouse-touch Down Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MULTI_DOWN );
   }

   /// Mouse-touch Down Event
   template <typename T>
   void
   callback_multi_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_DOWN, info));
   }

   /// Multi-touch Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MULTI_MOVE );
   }

   /// Multi-touch Move Event
   template <typename T>
   void
   callback_multi_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_MOVE, info));
   }

   /// Mouse-touch Up Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_MULTI_UP );
   }

   /// Mouse-touch Up Event
   template <typename T>
   void
   callback_multi_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_UP, info));
   }

   /// Resize Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_resize_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_RESIZE, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_RESIZE );
   }

   /// Resize Event
   template <typename T>
   void
   callback_resize_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_RESIZE, info));
   }

   /// Restack Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_restack_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_RESTACK, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_RESTACK );
   }

   /// Restack Event
   template <typename T>
   void
   callback_restack_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_RESTACK, info));
   }

   /// Show Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_show_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_SHOW, priority_,
            &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::object_smart, function_type>,
         EVAS_OBJECT_EVENT_SHOW );
   }

   /// Show Event
   template <typename T>
   void
   callback_show_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_SHOW, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_OBJECT_SMART_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas::object_smart::address_of<address_of>
      , ::eo_cxx::efl::gfx::base::address_of<address_of>
      , ::eo_cxx::efl::gfx::stack::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas::object::address_of<address_of>
      , ::eo_cxx::evas::signal_interface::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas::object_smart* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas::object_smart::address_const_of<address_const_of>
      , ::eo_cxx::efl::gfx::base::address_const_of<address_const_of>
      , ::eo_cxx::efl::gfx::stack::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas::object::address_const_of<address_const_of>
      , ::eo_cxx::evas::signal_interface::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas::object_smart const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_OBJECT_SMART_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas::object_smart) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas::object_smart>::value, "");

}


#include "evas_object_smart.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_OBJECT_SMART_HH

