#ifndef EFL_GENERATED_EVAS_TEXTGRID_HH
#define EFL_GENERATED_EVAS_TEXTGRID_HH

extern "C"
{
#include <Efl.h>
}
#include <Eo.hh>

#include <eo_cxx_interop.hh>

extern "C"
{
#include "evas_textgrid.eo.h"
}

#include "evas_object.eo.hh"
#include "efl_text_properties.eo.hh"

namespace evas {

struct textgrid;

}

/// @cond EO_CXX_ABSTRACT
namespace eo_cxx {

namespace evas {

struct textgrid
{
   /// @brief @brief Indicate for evas that part of a textgrid region (cells) has been updated.
   ///
   /// This function declares to evas that a region of cells was updated by
   /// code and needs refreshing. An application should modify cells like this
   /// as an example:
   ///
   /// @code
   /// Evas_Textgrid_Cell *cells;
   /// int i;
   ///
   /// cells = evas_object_textgrid_cellrow_get(obj, row);
   /// for (i = 0; i < width; i++) cells[i].codepoint = 'E';
   /// evas_object_textgrid_cellrow_set(obj, row, cells);
   /// evas_object_textgrid_update_add(obj, 0, row, width, 1);
   /// @endcode
   ///
   /// @see evas_object_textgrid_cellrow_set()
   /// @see evas_object_textgrid_cellrow_get()
   /// @see evas_object_textgrid_size_set()
   ///
   /// @since 1.7
   ///
   /// @param x The rect region of cells top-left x (column)
   /// @param y The rect region of cells top-left y (row)
   /// @param w The rect region size in number of cells (columns)
   /// @param h The rect region size in number of cells (rows)
   ///
   void update_add(int x_, int y_, int w_, int h_) const;

   /// @brief @brief Set the string at the given row of the given textgrid object.
   ///
   /// This function returns cells to the textgrid taken by
   /// evas_object_textgrid_cellrow_get(). The row pointer @p row should be the
   /// same row pointer returned by evas_object_textgrid_cellrow_get() for the
   /// same row @p y.
   ///
   /// @see evas_object_textgrid_cellrow_get()
   /// @see evas_object_textgrid_size_set()
   /// @see evas_object_textgrid_update_add()
   ///
   /// @since 1.7
   ///
   /// @param y The row index of the grid.
   /// @param row The string as a sequence of #Evas_Textgrid_Cell.
   ///
   void cellrow_set(int y_, const Evas_Textgrid_Cell * row_) const;

   /// @brief @brief Get the string at the given row of the given textgrid object.
   ///
   /// @return A pointer to the first cell of the given row.
   ///
   /// This function returns a pointer to the first cell of the line @p y
   /// of the textgrid object @p obj. If @p y is not between 0 and the
   /// number of lines of the grid - 1, or on error, this function return @c NULL.
   ///
   /// @see evas_object_textgrid_cellrow_set()
   /// @see evas_object_textgrid_size_set()
   /// @see evas_object_textgrid_update_add()
   ///
   /// @since 1.7
   ///
   /// @param y The row index of the grid.
   ///
   Evas_Textgrid_Cell * cellrow_get(int y_) const;

   /// @brief @brief The set color to the given palette at the given index of the given textgrid object.
   ///
   /// This function sets the color for the palette of type @p pal at the
   /// index @p idx of the textgrid object @p obj. The ARGB components are
   /// given by @p r, @p g, @p b and @p a. This color can be used when
   /// setting the #Evas_Textgrid_Cell structure. The components must set
   /// a pre-multiplied color. If pal is #EVAS_TEXTGRID_PALETTE_NONE or
   /// #EVAS_TEXTGRID_PALETTE_LAST, or if @p idx is not between 0 and 255,
   /// or on error, this function does nothing. The color components are
   /// clamped between 0 and 255. If @p idx is greater than the latest set
   /// color, the colors between this last index and @p idx - 1 are set to
   /// black (0, 0, 0, 0).
   ///
   /// @see evas_object_textgrid_palette_get()
   ///
   /// @since 1.7
   ///
   /// @param pal The type of the palette to set the color.
   /// @param idx The index of the paletter to which the color is stored.
   /// @param r The red component of the color.
   /// @param g The green component of the color.
   /// @param b The blue component of the color.
   /// @param a The alpha component of the color.
   ///
   void palette_set(Evas_Textgrid_Palette pal_, int idx_, int r_, int g_, int b_, int a_) const;

   /// @brief @brief The retrieve color to the given palette at the given index of the given textgrid object.
   ///
   /// This function retrieves the color for the palette of type @p pal at the
   /// index @p idx of the textgrid object @p obj. The ARGB components are
   /// stored in the buffers @p r, @p g, @p b and @p a. If @p idx is not
   /// between 0 and the index of the latest set color, or if @p pal is
   /// #EVAS_TEXTGRID_PALETTE_NONE or #EVAS_TEXTGRID_PALETTE_LAST, the
   /// values of the components are 0. @p r, @p g, @p b and @p a can be
   /// @c NULL.
   ///
   /// @see evas_object_textgrid_palette_set()
   ///
   /// @since 1.7
   ///
   /// @param pal The type of the palette to set the color.
   /// @param idx The index of the palette to which the color is stored.
   /// @param[out] r A pointer to the red component of the color.
   /// @param[out] g A pointer to the green component of the color.
   /// @param[out] b A pointer to the blue component of the color.
   /// @param[out] a A pointer to the alpha component of the color.
   ///
   void palette_get(Evas_Textgrid_Palette pal_, int idx_, int* r_, int* g_, int* b_, int* a_) const;

   /// @brief This function is not implemented yet, do not use.
   /// @return EVAS_TEXTGRID_FONT_STYLE_NORMAL
   ///
   /// @param styles 
   ///
   Evas_Textgrid_Font_Style supported_font_styles_get() const;

   /// @brief This function is not implemented yet, do not use.
   ///
   /// @param styles 
   ///
   void supported_font_styles_set(Evas_Textgrid_Font_Style styles_) const;

   /// @brief @brief Get the size of the textgrid object.
   ///
   /// This function retrieves the number of lines in the buffer @p
   /// h and the number of columns in the buffer @p w of
   /// the textgrid object @p obj. @p w or @p h can be
   /// @c NULL. On error, their value is 0.
   ///
   /// @since 1.7
   ///
   /// @param w The number of columns (width in cells) of the grid.
   /// @param h The number of rows (height in cells) of the grid.
   ///
   void size_get(int* w_, int* h_) const;

   /// @brief @brief Set the size of the textgrid object.
   ///
   /// This function sets the number of lines @p h and the number
   /// of columns @p w to the textgrid object @p obj. If
   /// @p w or @p h are less or equal than 0, this
   /// functiond does nothing.
   ///
   /// @since 1.7
   ///
   /// @param w The number of columns (width in cells) of the grid.
   /// @param h The number of rows (height in cells) of the grid.
   ///
   void size_set(int w_, int h_) const;

   /// @brief @brief Retrieve the size of a cell of the given textgrid object in pixels.
   ///
   /// This functions retrieves the width and height, in pixels, of a cell
   /// of the textgrid object @p obj and store them respectively in the
   /// buffers @p width and @p height. Their value depends on the
   /// monospace font used for the textgrid object, as well as the
   /// style. @p width and @p height can be @c NULL. On error, they are
   /// set to 0.
   ///
   /// @see evas_object_textgrid_font_set()
   /// @see evas_object_textgrid_supported_font_styles_set()
   ///
   /// @since 1.7
   ///
   /// @param width A pointer to the location to store the width in pixels of a cell.
   /// @param height A pointer to the location to store the height in
   /// pixels of a cell.
   ///
   void cell_size_get(int* width_, int* height_) const;



   static Eo_Class const* _eo_class()
   {
      return(EVAS_TEXTGRID_CLASS);
   }

   operator ::evas::textgrid() const;
   operator ::evas::textgrid&();
   operator ::evas::textgrid const&() const;

   /// @cond LOCAL
   template <typename D>
   struct address_of
   {
      operator ::evas::textgrid*() const { return static_cast<::evas::textgrid*>(static_cast<D const*>(this)->p); }
      operator ::evas::textgrid const*() const { return static_cast<::evas::textgrid const*>(static_cast<D const*>(this)->p); }
   };

   template <typename D>
   struct address_const_of
   {
      operator ::evas::textgrid const*() const { return static_cast<::evas::textgrid const*>(static_cast<D const*>(this)->p); }
   };
   /// @endcond

private:

   /// @internal
   Eo* _concrete_eo_ptr() const
   {
      return static_cast<::efl::eo::concrete const*>(static_cast<void const*>(this))->_eo_ptr();
   }

};

}

}
/// @endcond

namespace evas {

/// @brief Class textgrid
struct textgrid
      : ::efl::eo::concrete
      , EO_CXX_INHERIT(efl::gfx::base)
      , EO_CXX_INHERIT(efl::gfx::stack)
      , EO_CXX_INHERIT(efl::text_properties)
      , EO_CXX_INHERIT(eo::base)
      , EO_CXX_INHERIT(evas::common_interface)
      , EO_CXX_INHERIT(evas::object)
{
   //@{
   /**
      @brief Constructs a new evas::textgrid object.

      Constructs a new evas::textgrid object. If you want this object to be a child
      of another Eo object, use an @ref efl::eo::parent expression, like the example.

      Example:
      @code
      evas::textgrid my_textgrid(efl::eo::parent = parent_object);
      @endcode

      @see textgrid(Eo* eo)
   */
   explicit textgrid(::efl::eo::parent_type _p)
      : textgrid(_ctors_call(_p))
   {}

   explicit textgrid()
      : textgrid(_ctors_call(::efl::eo::parent = nullptr))
   {}
   //@}

   /// @brief Eo Constructor.
   ///
   /// Constructs the object from an Eo* pointer stealing its ownership.
   ///
   /// @param eo The Eo object pointer.
   ///
   explicit textgrid(Eo* eo)
      : ::efl::eo::concrete(eo)
   {}

   /// @brief nullptr_t Constructor.
   ///
   /// Constructs an empty (null) object.
   ///
   explicit textgrid(std::nullptr_t)
      : ::efl::eo::concrete(nullptr)
   {}

   /// @brief Copy Constructor.
   ///
   textgrid(textgrid const& other)
      : textgrid(eo_ref(other._eo_ptr()))
   {}

   ~textgrid() {}

   /// @brief @brief Indicate for evas that part of a textgrid region (cells) has been updated.
   ///
   /// This function declares to evas that a region of cells was updated by
   /// code and needs refreshing. An application should modify cells like this
   /// as an example:
   ///
   /// @code
   /// Evas_Textgrid_Cell *cells;
   /// int i;
   ///
   /// cells = evas_object_textgrid_cellrow_get(obj, row);
   /// for (i = 0; i < width; i++) cells[i].codepoint = 'E';
   /// evas_object_textgrid_cellrow_set(obj, row, cells);
   /// evas_object_textgrid_update_add(obj, 0, row, width, 1);
   /// @endcode
   ///
   /// @see evas_object_textgrid_cellrow_set()
   /// @see evas_object_textgrid_cellrow_get()
   /// @see evas_object_textgrid_size_set()
   ///
   /// @since 1.7
   ///
   /// @param x The rect region of cells top-left x (column)
   /// @param y The rect region of cells top-left y (row)
   /// @param w The rect region size in number of cells (columns)
   /// @param h The rect region size in number of cells (rows)
   ///
   void update_add(int x_, int y_, int w_, int h_) const;

   /// @brief @brief Set the string at the given row of the given textgrid object.
   ///
   /// This function returns cells to the textgrid taken by
   /// evas_object_textgrid_cellrow_get(). The row pointer @p row should be the
   /// same row pointer returned by evas_object_textgrid_cellrow_get() for the
   /// same row @p y.
   ///
   /// @see evas_object_textgrid_cellrow_get()
   /// @see evas_object_textgrid_size_set()
   /// @see evas_object_textgrid_update_add()
   ///
   /// @since 1.7
   ///
   /// @param y The row index of the grid.
   /// @param row The string as a sequence of #Evas_Textgrid_Cell.
   ///
   void cellrow_set(int y_, const Evas_Textgrid_Cell * row_) const;

   /// @brief @brief Get the string at the given row of the given textgrid object.
   ///
   /// @return A pointer to the first cell of the given row.
   ///
   /// This function returns a pointer to the first cell of the line @p y
   /// of the textgrid object @p obj. If @p y is not between 0 and the
   /// number of lines of the grid - 1, or on error, this function return @c NULL.
   ///
   /// @see evas_object_textgrid_cellrow_set()
   /// @see evas_object_textgrid_size_set()
   /// @see evas_object_textgrid_update_add()
   ///
   /// @since 1.7
   ///
   /// @param y The row index of the grid.
   ///
   Evas_Textgrid_Cell * cellrow_get(int y_) const;

   /// @brief @brief The set color to the given palette at the given index of the given textgrid object.
   ///
   /// This function sets the color for the palette of type @p pal at the
   /// index @p idx of the textgrid object @p obj. The ARGB components are
   /// given by @p r, @p g, @p b and @p a. This color can be used when
   /// setting the #Evas_Textgrid_Cell structure. The components must set
   /// a pre-multiplied color. If pal is #EVAS_TEXTGRID_PALETTE_NONE or
   /// #EVAS_TEXTGRID_PALETTE_LAST, or if @p idx is not between 0 and 255,
   /// or on error, this function does nothing. The color components are
   /// clamped between 0 and 255. If @p idx is greater than the latest set
   /// color, the colors between this last index and @p idx - 1 are set to
   /// black (0, 0, 0, 0).
   ///
   /// @see evas_object_textgrid_palette_get()
   ///
   /// @since 1.7
   ///
   /// @param pal The type of the palette to set the color.
   /// @param idx The index of the paletter to which the color is stored.
   /// @param r The red component of the color.
   /// @param g The green component of the color.
   /// @param b The blue component of the color.
   /// @param a The alpha component of the color.
   ///
   void palette_set(Evas_Textgrid_Palette pal_, int idx_, int r_, int g_, int b_, int a_) const;

   /// @brief @brief The retrieve color to the given palette at the given index of the given textgrid object.
   ///
   /// This function retrieves the color for the palette of type @p pal at the
   /// index @p idx of the textgrid object @p obj. The ARGB components are
   /// stored in the buffers @p r, @p g, @p b and @p a. If @p idx is not
   /// between 0 and the index of the latest set color, or if @p pal is
   /// #EVAS_TEXTGRID_PALETTE_NONE or #EVAS_TEXTGRID_PALETTE_LAST, the
   /// values of the components are 0. @p r, @p g, @p b and @p a can be
   /// @c NULL.
   ///
   /// @see evas_object_textgrid_palette_set()
   ///
   /// @since 1.7
   ///
   /// @param pal The type of the palette to set the color.
   /// @param idx The index of the palette to which the color is stored.
   /// @param[out] r A pointer to the red component of the color.
   /// @param[out] g A pointer to the green component of the color.
   /// @param[out] b A pointer to the blue component of the color.
   /// @param[out] a A pointer to the alpha component of the color.
   ///
   void palette_get(Evas_Textgrid_Palette pal_, int idx_, int* r_, int* g_, int* b_, int* a_) const;

   /// @brief This function is not implemented yet, do not use.
   /// @return EVAS_TEXTGRID_FONT_STYLE_NORMAL
   ///
   /// @param styles 
   ///
   Evas_Textgrid_Font_Style supported_font_styles_get() const;

   /// @brief This function is not implemented yet, do not use.
   ///
   /// @param styles 
   ///
   void supported_font_styles_set(Evas_Textgrid_Font_Style styles_) const;

   /// @brief @brief Get the size of the textgrid object.
   ///
   /// This function retrieves the number of lines in the buffer @p
   /// h and the number of columns in the buffer @p w of
   /// the textgrid object @p obj. @p w or @p h can be
   /// @c NULL. On error, their value is 0.
   ///
   /// @since 1.7
   ///
   /// @param w The number of columns (width in cells) of the grid.
   /// @param h The number of rows (height in cells) of the grid.
   ///
   void size_get(int* w_, int* h_) const;

   /// @brief @brief Set the size of the textgrid object.
   ///
   /// This function sets the number of lines @p h and the number
   /// of columns @p w to the textgrid object @p obj. If
   /// @p w or @p h are less or equal than 0, this
   /// functiond does nothing.
   ///
   /// @since 1.7
   ///
   /// @param w The number of columns (width in cells) of the grid.
   /// @param h The number of rows (height in cells) of the grid.
   ///
   void size_set(int w_, int h_) const;

   /// @brief @brief Retrieve the size of a cell of the given textgrid object in pixels.
   ///
   /// This functions retrieves the width and height, in pixels, of a cell
   /// of the textgrid object @p obj and store them respectively in the
   /// buffers @p width and @p height. Their value depends on the
   /// monospace font used for the textgrid object, as well as the
   /// style. @p width and @p height can be @c NULL. On error, they are
   /// set to 0.
   ///
   /// @see evas_object_textgrid_font_set()
   /// @see evas_object_textgrid_supported_font_styles_set()
   ///
   /// @since 1.7
   ///
   /// @param width A pointer to the location to store the width in pixels of a cell.
   /// @param height A pointer to the location to store the height in
   /// pixels of a cell.
   ///
   void cell_size_get(int* width_, int* height_) const;

   /// A callback was added.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_add_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_ADD, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EO_BASE_EVENT_CALLBACK_ADD );
   }

   /// A callback was added.
   template <typename T>
   void
   callback_callback_add_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_ADD, info));
   }

   /// A callback was deleted.
   template <typename F>
   ::efl::eo::signal_connection
   callback_callback_del_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EO_BASE_EVENT_CALLBACK_DEL, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EO_BASE_EVENT_CALLBACK_DEL );
   }

   /// A callback was deleted.
   template <typename T>
   void
   callback_callback_del_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EO_BASE_EVENT_CALLBACK_DEL, info));
   }

   /// Size hints changed event
   template <typename F>
   ::efl::eo::signal_connection
   callback_changed_size_hints_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS );
   }

   /// Size hints changed event
   template <typename T>
   void
   callback_changed_size_hints_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_CHANGED_SIZE_HINTS, info));
   }

   /// Focus In Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_focus_in_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FOCUS_IN, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_FOCUS_IN );
   }

   /// Focus In Event
   template <typename T>
   void
   callback_focus_in_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FOCUS_IN, info));
   }

   /// Focus Out Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_focus_out_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FOCUS_OUT, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_FOCUS_OUT );
   }

   /// Focus Out Event
   template <typename T>
   void
   callback_focus_out_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FOCUS_OUT, info));
   }

   /// Object Being Freed (Called after Del)
   template <typename F>
   ::efl::eo::signal_connection
   callback_free_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_FREE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_FREE );
   }

   /// Object Being Freed (Called after Del)
   template <typename T>
   void
   callback_free_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_FREE, info));
   }

   /// Hide Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_hide_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_HIDE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_HIDE );
   }

   /// Hide Event
   template <typename T>
   void
   callback_hide_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_HIDE, info));
   }

   /// Events go on/off hold
   template <typename F>
   ::efl::eo::signal_connection
   callback_hold_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_HOLD, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_HOLD );
   }

   /// Events go on/off hold
   template <typename T>
   void
   callback_hold_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_HOLD, info));
   }

   /// Image has been preloaded
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_preloaded_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_PRELOADED, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_PRELOADED );
   }

   /// Image has been preloaded
   template <typename T>
   void
   callback_image_preloaded_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_PRELOADED, info));
   }

   /// Image resize
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_resize_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_RESIZE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_RESIZE );
   }

   /// Image resize
   template <typename T>
   void
   callback_image_resize_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_RESIZE, info));
   }

   /// Image data has been unloaded (by some mechanism in Evas that throw out original image data)
   template <typename F>
   ::efl::eo::signal_connection
   callback_image_unloaded_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_IMAGE_UNLOADED, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_IMAGE_UNLOADED );
   }

   /// Image data has been unloaded (by some mechanism in Evas that throw out original image data)
   template <typename T>
   void
   callback_image_unloaded_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_IMAGE_UNLOADED, info));
   }

   /// Key Press Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_key_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_KEY_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_KEY_DOWN );
   }

   /// Key Press Event
   template <typename T>
   void
   callback_key_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_KEY_DOWN, info));
   }

   /// Key Release Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_key_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_KEY_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_KEY_UP );
   }

   /// Key Release Event
   template <typename T>
   void
   callback_key_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_KEY_UP, info));
   }

   /// Mouse Button Down Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_DOWN );
   }

   /// Mouse Button Down Event
   template <typename T>
   void
   callback_mouse_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_DOWN, info));
   }

   /// Mouse In Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_in_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_IN, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_IN );
   }

   /// Mouse In Event
   template <typename T>
   void
   callback_mouse_in_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_IN, info));
   }

   /// Mouse Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_MOVE );
   }

   /// Mouse Move Event
   template <typename T>
   void
   callback_mouse_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_MOVE, info));
   }

   /// Mouse Out Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_out_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_OUT, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_OUT );
   }

   /// Mouse Out Event
   template <typename T>
   void
   callback_mouse_out_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_OUT, info));
   }

   /// Mouse Button Up Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_UP );
   }

   /// Mouse Button Up Event
   template <typename T>
   void
   callback_mouse_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_UP, info));
   }

   /// Mouse Wheel Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_mouse_wheel_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOUSE_WHEEL, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOUSE_WHEEL );
   }

   /// Mouse Wheel Event
   template <typename T>
   void
   callback_mouse_wheel_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOUSE_WHEEL, info));
   }

   /// Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MOVE );
   }

   /// Move Event
   template <typename T>
   void
   callback_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MOVE, info));
   }

   /// Mouse-touch Down Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_down_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_DOWN, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MULTI_DOWN );
   }

   /// Mouse-touch Down Event
   template <typename T>
   void
   callback_multi_down_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_DOWN, info));
   }

   /// Multi-touch Move Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_move_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_MOVE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MULTI_MOVE );
   }

   /// Multi-touch Move Event
   template <typename T>
   void
   callback_multi_move_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_MOVE, info));
   }

   /// Mouse-touch Up Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_multi_up_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_MULTI_UP, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_MULTI_UP );
   }

   /// Mouse-touch Up Event
   template <typename T>
   void
   callback_multi_up_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_MULTI_UP, info));
   }

   /// Resize Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_resize_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_RESIZE, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_RESIZE );
   }

   /// Resize Event
   template <typename T>
   void
   callback_resize_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_RESIZE, info));
   }

   /// Restack Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_restack_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_RESTACK, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_RESTACK );
   }

   /// Restack Event
   template <typename T>
   void
   callback_restack_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_RESTACK, info));
   }

   /// Show Event
   template <typename F>
   ::efl::eo::signal_connection
   callback_show_add(F && callback_,
                        ::efl::eo::callback_priority priority_ =
                        ::efl::eo::callback_priorities::default_)
   {
      typedef typename std::remove_reference<F>::type function_type;
      ::std::unique_ptr<function_type> f ( new function_type(std::forward<F>(callback_)) );
      eo_do(_concrete_eo_ptr(),
            eo_event_callback_priority_add
            (EVAS_OBJECT_EVENT_SHOW, priority_,
            &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>, f.get()));
      return ::efl::eo::make_signal_connection
         (f, _concrete_eo_ptr(), &::efl::eo::_detail::event_callback<::evas::textgrid, function_type>,
         EVAS_OBJECT_EVENT_SHOW );
   }

   /// Show Event
   template <typename T>
   void
   callback_show_call(T* info)
   {
      eo_do(_concrete_eo_ptr(), eo_event_callback_call
            (EVAS_OBJECT_EVENT_SHOW, info));
   }



   static Eo_Class const* _eo_class()
   {
      return(EVAS_TEXTGRID_CLASS);
   }

   /// @cond LOCAL
   struct address_of
      : ::eo_cxx::evas::textgrid::address_of<address_of>
      , ::eo_cxx::efl::gfx::base::address_of<address_of>
      , ::eo_cxx::efl::gfx::stack::address_of<address_of>
      , ::eo_cxx::efl::text_properties::address_of<address_of>
      , ::eo_cxx::eo::base::address_of<address_of>
      , ::eo_cxx::evas::common_interface::address_of<address_of>
      , ::eo_cxx::evas::object::address_of<address_of>
      , ::efl::eo::detail::concrete_address_of
   {
      explicit address_of(::evas::textgrid* p)
         : ::efl::eo::detail::concrete_address_of(p)
      {}
   };
   address_of operator&() { return address_of(this); }

   struct address_const_of
      : ::eo_cxx::evas::textgrid::address_const_of<address_const_of>
      , ::eo_cxx::efl::gfx::base::address_const_of<address_const_of>
      , ::eo_cxx::efl::gfx::stack::address_const_of<address_const_of>
      , ::eo_cxx::efl::text_properties::address_const_of<address_const_of>
      , ::eo_cxx::eo::base::address_const_of<address_const_of>
      , ::eo_cxx::evas::common_interface::address_const_of<address_const_of>
      , ::eo_cxx::evas::object::address_const_of<address_const_of>
      , ::efl::eo::detail::concrete_address_const_of
   {
      explicit address_const_of(::evas::textgrid const* p)
         : ::efl::eo::detail::concrete_address_const_of(p)
      {}
   };
   address_const_of operator&() const { return address_const_of(this); }

   /// @endcond

private:

   /// @internal
   static Eo* _ctors_call(::efl::eo::parent_type _p)
   {
      Eo* _ret_eo = eo_add_ref(EVAS_TEXTGRID_CLASS, _p._eo_raw);

      return _ret_eo;
   }

   /// @internal
   Eo* _concrete_eo_ptr() const { return _eo_ptr(); }
};

static_assert(sizeof(::evas::textgrid) == sizeof(Eo*), "");
static_assert(std::is_standard_layout<::evas::textgrid>::value, "");

}


#include "evas_textgrid.eo.impl.hh"

#endif // EFL_GENERATED_EVAS_TEXTGRID_HH

